package ucot.core;

import java.net.URL;
import java.util.Vector;

import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.output.OutputInterface;
import ucot.parser.ParserInterface;
/**
 * This interface controls the basics of the UCOT-program
 * (starting, parsing, shuting down etc).
 */
public interface ControlInterface {
	/**
	 * Shuts down the program.
	 *
	 */
	public void shutdown();
	
	/**
	 * Clears the whole analyze model.
	 */
	public void clearAnalyzeModel();
	
	/**
	 * Loads usecases from file
	 * @param url url of the file
	 */
	public void loadUseCases(URL url);
	
	/**
	 * Loads analyze model from a file
	 * @param url url of the file containing analyze model
	 */
	public void loadAnalyzeModel(URL url);
	
	/**
	 * Saves analyze model to a file
	 * @param url target file
	 */
	public void saveAnalyzeModel(URL url);
	
	/**
	 * Returns all loaded usecases
	 * @return use case colletion
	 */
	public UseCaseCollection getUseCaseCollection();
	
	/**
	 * Requests core to parse use case, perform heuristic on it and add it to given analyze model
	 * @param useCase use case to work magic on
	 * @param model analyze model where specified usecase is added
	 */
	public void addToAnalyzeModel(UseCase usecase) throws Exception;
	
	/**
	 * Requests core to parse use case, perform heuristic on it and add it to given analyze model
	 * @param useCase use case to work magic on
	 * @param model analyze model where specified usecase is added
	 * @param parser Parser to use
	 */
	public void addToAnalyzeModel(UseCase usecase, ParserInterface parser) throws Exception;
	
	/**
	 * Requests core to parse use case, perform heuristic on it and add it to given analyze model
	 * @param useCase use case to work magic on
	 * @param model analyze model where specified usecase is added
	 * @param parser Parser to use
	 * @param heuristic heuristic to use
	 */
	public void addToAnalyzeModel(UseCase usecase, ParserInterface parser, HeuristicInterface heuristic) throws Exception;
	
	
	/**
	 * returns vector of all analyze models from CORE
	 * @return specified analyze model
	 */
	public AnalyzeModel getAnalyzeModel();	
	
	/**
	 * Returns a vector of parsers that are available
	 * @return vector containing parser adapters
	 */
	public Vector<ParserInterface> getParsers();
	
	/**
	 * Returns a list of heuristics that are available
	 * @return vector containing heuristic adapter
	 */
	public Vector<HeuristicInterface> getHeuristics();
	
	/**
	 * Returns a list of output adapters available.
	 * @return vector containing output adapter names
	 */
	public Vector<OutputInterface> getOutputs();
	
	/**
	 * Returns InputCollection of inputs.
	 * @return InputCollection
	 */
	public InputCollection getInputs();
	
	/**
	 * Sets the default parser to use.
	 * @param parser Parser to be used by default
	 */
	public void setCurrentParser(ParserInterface parser);
	
	/**
	 * Sets the default heuristic to use.
	 * @param heuristic default heuristic to be used
	 */
	public void setCurrentHeuristic(HeuristicInterface heuristic);
	
	/**
	 * returns the current default parser
	 * @return default parser
	 */
	public ParserInterface getCurrentParser();
	
	/**
	 * returns the current heuristic
	 * @return current heuristic
	 */
	public HeuristicInterface getCurrentHeuristic();
	
	/**
	 * Exports given analyze model to given url using given outputadapter
	 * @param url destination url
	 * @param output output adapter
	 * @param model analyzemodel to export
	 */
	public void output(URL url, OutputInterface output, AnalyzeModel model) throws Exception;	
}
