package ucot.core;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;
import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.sourceforge.gxl.GXL;

import ucot.Messages;
import ucot.heuristic.AbbottsHeuristic;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.InputInterface;
import ucot.input.ProcessMlInputAdapter;
import ucot.input.SimpleInputAdapter;
import ucot.input.UseCase;
import ucot.input.UseCaseCollection;
import ucot.model.AnalyzeModel;
import ucot.model.relation.RelationalAnalyzeModel;
import ucot.output.GXLOutput;
import ucot.output.Output;
import ucot.output.OutputInterface;
import ucot.parser.ParsedData;
import ucot.parser.ParserInterface;
import ucot.parser.SimpleInputParser;
import ucot.parser.StanfordAdapter;
import ucot.ui.UIInterface;
import ucot.ui.gui.GraphicalUI;

/**
 * This is the implementation of core of the program,
 * the controller of the program.
 * 
 *
 */
public class Core implements ControlInterface {

	private InputCollection inputs = new InputCollection();
	private UseCaseCollection usecases = new UseCaseCollection();
	private AnalyzeModel model = new RelationalAnalyzeModel();
	private Vector<HeuristicInterface> heuristics = new Vector<HeuristicInterface>();
	private Vector<ParserInterface> parsers = new Vector<ParserInterface>();
	private Vector<OutputInterface> outputs = new Vector<OutputInterface>();
	private UIInterface ui = null;
	private ParserInterface currentParser = null;
	private HeuristicInterface currentHeuristic = null;

	public Core(){		
		
		// Set up simple logging system.
		ConsoleHandler cs = new ConsoleHandler();
		System.out.println(Logger.getLogger("ucot").getHandlers().length);
		cs.setLevel(Level.ALL);
		Logger.getLogger("ucot").addHandler(cs);
		Logger.getLogger("ucot").setLevel(Level.ALL);
		Logger.getLogger("ucot").setUseParentHandlers(false);
		
		
		// Inform user that program is starting.
		System.out.println("Launching UCOT-core");
		
		// Set defaults.
        currentParser = new StanfordAdapter();
		currentHeuristic = new AbbottsHeuristic();
		 
		inputs.add(new SimpleInputAdapter());
		inputs.add(new ProcessMlInputAdapter());
		
		outputs.add(new GXLOutput());
		outputs.add(new Output());
		
		heuristics.add(currentHeuristic);
        parsers.add(new SimpleInputParser());
        parsers.add(currentParser);
        
        ui = new GraphicalUI(this);
	}
	
	
	/**
	 * Main method that starts the core.
	 * @param args
	 */
	public static void main(String args[]){
		Core core = new Core();
	}
	
	
// ------------ CONTROL INTERFACE -------------
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#shutdown()
	 */
	public void shutdown() {
		//Get rid of usecases, inputs, and such
		inputs = null;
		usecases = null;
		model = null;
		heuristics = null;
		parsers = null;
		outputs = null;		
		currentParser = null;
		currentHeuristic = null;
	}


	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#loadUseCases(java.net.URL)
	 */
	public void loadUseCases(URL url) {
		if (url == null)
			return;
		
        InputInterface input = inputs.getInputForUrl(url);
        
        if (input == null) {
        	if ( ui != null ) ui.printError(Messages.getString("Core.1")); //$NON-NLS-1$
        	return;
        }
        
        try {            
        		this.usecases.merge(input.read(url));
        } catch (IOException e) {
                System.err.println(e.getMessage());
        }       
        
        if (ui != null)
        	ui.useCasesLoaded();
    }

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#loadAnalyzeModel(java.net.URL)
	 */
	public void loadAnalyzeModel(URL url) {
		if (ui != null)
        	ui.analyzeModelLoaded();
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getUseCaseCollection()
	 */
	public UseCaseCollection getUseCaseCollection() {		
		return usecases;
	}
	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#clearAnalyzeModel()
	 */
	public void clearAnalyzeModel() {
		model.clear();
		usecases.markAllUnanalyzed();
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#addToAnalyzeModel(ucot.input.UseCase, ucot.parser.ParserInterface, ucot.heuristic.HeuristicInterface)
	 */
	public void addToAnalyzeModel(UseCase usecase, ParserInterface parser, HeuristicInterface heuristic) throws Exception {
		if (heuristic == null || parser == null)
			return;
		
		AnalyzeModel temp = new RelationalAnalyzeModel();
		
		try {
			ParsedData data = parser.parse(usecase);
			heuristic.doHeuristic(data,temp);
			usecase.setAnalyzeModel(temp);
			model.getEditor().merge(temp);
			usecase.setUseCaseAnalyzed();
			if (ui != null)
	        	ui.useCaseAdded();
		} catch(Exception e) {
			e.printStackTrace();
		}
        
	}
	

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#addToAnalyzeModel(ucot.input.UseCase, ucot.parser.ParserInterface)
	 */
	public void addToAnalyzeModel(UseCase usecase, ParserInterface parser) throws Exception {
		addToAnalyzeModel(usecase,parser,currentHeuristic);
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#addToAnalyzeModel(ucot.input.UseCase)
	 */
	public void addToAnalyzeModel(UseCase usecase) throws Exception {
		addToAnalyzeModel(usecase,currentParser,currentHeuristic);
	}

	
	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getAnalyzeModel()
	 */
	public AnalyzeModel getAnalyzeModel() {		
		return model;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getParsers()
	 */
	public Vector<ParserInterface> getParsers() {		
		return parsers;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getHeuristics()
	 */
	public Vector<HeuristicInterface> getHeuristics() {
		return heuristics;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getOutputs()
	 */
	public Vector<OutputInterface> getOutputs() { 
		return outputs;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getInputs()
	 */
	public InputCollection getInputs() {
		return inputs;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#setCurrentParser(ucot.parser.ParserInterface)
	 */
	public void setCurrentParser(ParserInterface parser) {
		currentParser = parser;
		
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#setCurrentHeuristic(ucot.heuristic.HeuristicInterface)
	 */
	public void setCurrentHeuristic(HeuristicInterface heuristic) {
		currentHeuristic = heuristic;
		
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getCurrentParser()
	 */
	public ParserInterface getCurrentParser() {
		return currentParser;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#getCurrentHeuristic()
	 */
	public HeuristicInterface getCurrentHeuristic() {
		return currentHeuristic;
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#output(java.net.URL, ucot.output.OutputInterface, ucot.model.AnalyzeModel)
	 */
	public void output(URL url, OutputInterface output, AnalyzeModel model) throws Exception {
		if (output != null)
			output.output(model,url);
		if (ui != null)
			ui.exportDone();
	}

	/* (non-Javadoc)
	 * @see ucot.core.ControlInterface#saveAnalyzeModel(java.net.URL)
	 */
	public void saveAnalyzeModel(URL url) {
		
	}

	
}
