package ucot.input;

import java.net.URL;
import java.util.Iterator;
import java.util.Vector;

import ucot.heuristic.HeuristicInterface;
import ucot.model.AnalyzeModel;
import ucot.parser.ParserInterface;

/**
 * UseCase class. Contains UseCaseSteps related to this usecase.
 * @author vevijopi
 *
 */
public class UseCase implements Iterable<UseCaseStep>{

	/**
	 * Vector that contains all usecase steps
	 */
	private Vector<UseCaseStep> useCaseSteps;	
	private URL url;
	private String name;
	/**
	 * Ise usecase added to current analyze model.
	 */
    private boolean isUseCaseAnalyzed;
    
    /**
     * Is this a sub-usecase, mainly used when connecting sub-usecases to usecase steps
     */
	private boolean subUseCase = false;
	
	/**
	 * When heuristic was ran on this usecase, this model was created.
	 * It was later added to the main usecase, but a copy was left.
	 * This copy is used for highlighting. 
	 */
	private AnalyzeModel model = null;
	
	/**
	 * Which heuristicInterface was used on this usecase
	 */
	private HeuristicInterface heuristic = null;
	
	/**
	 * Which parserInterface was used on this usecase
	 */
	private ParserInterface parser = null;
	
	/**
	 * UseCase's id loaded from processml files or from simple usecase format
	 * Only used for connecting usecase steps to sub-usecases
	 */
	private String useCaseId = null;	
	
	
    public UseCase() {
        useCaseSteps = new Vector<UseCaseStep>();        
        url = null;
        useCaseId = new String();
        this.isUseCaseAnalyzed = false;
    }
    
    /**
     * Set the parser that was used to parse this usecase
     * @param parser parser that was used
     */
    public void setParser(ParserInterface parser) {
    	this.parser = parser;
    }
    
    /**
     * Set the heuristic that was used to this usecaes
     * @param heuristic heuristic
     */
    public void setHeuristic(HeuristicInterface heuristic) {
    	this.heuristic = heuristic;
    }
    
    /**
     * Get the parser that was used on this usecase
     * @return
     */
    public ParserInterface getParser() {
    	return parser;
    }
    
    /**
     * Get the heuristic that was used on this usecase
     * @return
     */
    public HeuristicInterface getHeuristic() {
    	return heuristic;
    }
    
    /**
     * Returns name of the parser that was used in creating this usecase
     * @return
     */
    public String getParserName() {
    	if (parser == null)
    		return "";
    	return parser.getName();
    }
    
    /**
     * Returns name of the heuristic that was used in creating this usecase
     * @return
     */
    public String getHeuristicName() {
    	if (heuristic == null)
    		return "";
    	return heuristic.getName();
    }

    
    /**
     * Returns the (mini) analyzemodel that was created from this (and only this) usecase
     * @return
     */
    public AnalyzeModel getAnalyzeModel() {
    	return model;
    }
    
    /**
     * Sets the (mini) analyzemodel that was created from this (and only this) usecase
     * @param a
     */
    public void setAnalyzeModel(AnalyzeModel model) {
    	this.model = model;
    }
    
    /**
     * Mark this usecase as a sub usecase
     * @param sub true if this usecase is a sub-usecase
     */
    public void setAsSubUseCase(boolean sub) {
    	subUseCase = sub;
    }
    
    /**
     * Is this usecase a sub-usecaes
     * @return true if it is
     */
    public boolean isSubUseCase() {
    	return subUseCase;
    }
        
    /**
     * Is this usecase analyzed and added to main analyzemodel
     * @return true if it is
     */
    public boolean isUseCaseAnalyzed() {
        return isUseCaseAnalyzed;
    }

    /**
     * Set wheiter this usecase is analyzed.
     * @param analyzed is the usecase analyzed
     */
    public void setUseCaseAnalyzed(boolean analyzed) {
        this.isUseCaseAnalyzed = analyzed;
    }

    /**
     * mark this usecase as anlyzed
     *
     */
    public void setUseCaseAnalyzed() {
        setUseCaseAnalyzed(true);
    }

    /**
     * Iterator for the usecase steps
     */
    public Iterator<UseCaseStep> iterator(){
    	return new Iterator<UseCaseStep>(){
    		int index = 0;
			public boolean hasNext() {
				return index < getStepCount();
			}

			public UseCaseStep next() {				
				return getStep(index++);
			}

			public void remove(){
				throw new UnsupportedOperationException();
			}	
    		
    	};
    }
    
    /**
     * Set relation id for this usecase. Only used for connecting UseCaseSteps to subusecases
     * @param relationId this usecase's id loaded from file 
     */
    public void setId(String relationId) {
    	useCaseId = relationId;    	 
    }
    
    /**
     * Returns this usecase's id.
     * @return usecase id
     */
    public String getUseCaseId() {
    	return useCaseId;
    }
    
    /**
     * Returns the count of steps this usecase has
     * @return step count
     */
	public int getStepCount() {
		return useCaseSteps.size();
	}
	
	/**
	 * Returns step with given index
	 * @param index index of the step
	 * @return UseCaseStep or null if index was incorrect
	 */
	public UseCaseStep getStep(int index) {
		if (index < 0 || index >= getStepCount())
			return null;
		return useCaseSteps.get(index);
	}
	
	/**
	 * Removes all usecase's steps
	 *
	 */
	public void clear()	{
		useCaseSteps.clear();
	}
	
	
	/**
	 * Adds a new step to this usecase
	 * @param step step's description
	 * @param subUseCase reference to sub usecase
	 */
	public void addStep(String step, UseCase subUseCase) {
		useCaseSteps.add(new UseCaseStep(step,subUseCase));		
	}	
	
	/**
	 * Adds a new UseCaseStep to this usecase
	 * @param step UseCaseStep object to add
	 */
	public void addStep(UseCaseStep step) {
		if (step != null)
			useCaseSteps.add(step);		
	}
	
	/**
	 * Returns the url where this usecase was loaded from
	 * @return
	 */
	public URL getUrl() {
		return url;
	}
	
	/**
	 * Sets the url where this usecase was loaded from
	 * @param url
	 */
	public void setUrl(URL url) {
		this.url = url;
	}
	
	/**
	 * Returns this usecase's name
	 * @return name
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets a new name for this usecase
	 * @param name new name
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	
	/**
	 * Returns this usecase as a string with this format:
	 * Name
	 * ---
	 * (step index). (step)
	 * (step index). (step)
	 * ...
	 * 
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(name + "\n---\n");		
		for(int i=0;i<getStepCount();i++) {			
			sb.append(Integer.toString(i) + ". " + getStep(i) + "\n");
		}		
		return sb.toString();
	}
	
	/**
	 * Equals method for usecases.
	 * For now, we don't care about sub-usecases. Should we?
	 */
	public boolean equals(Object obj){
		if (obj instanceof UseCase) {
			UseCase other = (UseCase) obj;
			boolean result = (other.getStepCount() == getStepCount());
			for ( int i = 0; i < getStepCount() && i < other.getStepCount() && result; i++ ) {
				if ( !getStep(i).equals(other.getStep(i)) ) result = false;
			}
			return this.getName().equals(other.getName()) && result;
		}
		return false;
	}
	
	
}
