package ucot.input;

import java.net.URL;
import java.util.List;
import java.util.Observable;
import java.util.Vector;

/**
 * Collection of usecases. Handles finds and merges for them.
 * @author vevijopi
 *
 */
public class UseCaseCollection extends Observable{
	private Vector<UseCase> useCases;
	
	
	public UseCaseCollection() {
		useCases = new Vector<UseCase>();
	}
	
	/**
	 * Merges given UseCaseCollection to this 
	 * @param collection UseCaseCollection to merge
	 */
	public synchronized void merge(UseCaseCollection collection) {
		

		if (collection != null) {
			synchronized(collection){
				//Go through all usecases from the another collection
				for(int i=0;i<collection.getUseCaseCount();i++) {
					//Check wheither each usecase exists in this collection
					if (!exists(collection.getUseCase(i)))
						//If not, add
						addUseCase(collection.getUseCase(i));
				}
			}
		}
        setChanged();
        notifyObservers();
	}
	
	/**
	 * Checks if given UseCase exists in this collection
	 * @param usecase UseCase to test
	 * @return true if UseCase exists in this collection
	 */
	public boolean exists(UseCase usecase) {
		return useCases.contains(usecase);
	}
	
	/**
	 * Returns the count of use cases inside this collection.
	 * @return The count of use cases.
	 */
	public int getUseCaseCount() {
		return useCases.size();
	}

    /**
     * Add usecase to collection and notify observers
     * @param usecase UseCase to add
     */
	public void addUseCase(UseCase usecase) {
		useCases.add(usecase);
        setChanged();
        notifyObservers();
	}
	
	/**
	 * Finds usecase that has the given id
	 * @param id id to find
	 * @return usecase with given id, -1 if none found
	 */
	public UseCase find(String id) {		
		for(int i=0;i<getUseCaseCount();i++) {			
			if (getUseCase(i).getUseCaseId().equals(id))
				return getUseCase(i);			
		}
		
		return null;
	}
	
    /**
     * Returns UseCase from index
     * @param index
     * @return UseCase
     */
	public UseCase getUseCase(int index) {
		return useCases.get(index);
	}
	
    /**
     * Returns list of source URLs
     * @return
     */
    public List<URL> getURLs() {
        
        List<URL> urls = new Vector<URL>();
        
        for (UseCase u: useCases) {

            // if url is not on the list, add it
            if (! urls.contains(u.getUrl()) )  {
                urls.add(u.getUrl());
            }
        }
        return urls;
    }
    
    /**
     * Returns list of usecases from source described in url.
     * @param url
     * @return
     */
    public List<UseCase> getUseCasesFromURL(URL url) {
        List<UseCase> cases = new Vector<UseCase>();
        
        for (UseCase u: useCases) {
            
            if ( u.getUrl().equals(url) ) {
                cases.add(u);
            }
        }
        
        return cases;
    }
    
    /**
     * Clears loaded usecases.
     *
     */
	public void clear(){
		useCases.clear();
        setChanged();
        notifyObservers();
	}
    
	/**
	 * Removes usecase with given index
	 * @param index index of the usecase to remove
	 */
	public void remove(int index) {
		useCases.remove(index);
	}
	
	/**
	 * Removes all usecases that have been marked as subusecase
	 *
	 */
	public void removeAllSubUseCase() {
		for(int i=0;i<useCases.size();i++) {
			if (getUseCase(i).isSubUseCase()) {
				remove(i);
				i--;
			}
		}
	}
	
	/**
	 * Marks all usecases within this collection as unanalyzed
	 *
	 */
	public void markAllUnanalyzed() {
		for ( int i=0; i < useCases.size(); i++ ) {
			markAllUnanalyzed(useCases.get(i));
        }
		setChanged();
        notifyObservers();
	}
	
	/**
	 * Marks given usecase and it's sub usecases as unanalyzed
	 * @param usecase usecase to be marked
	 */
    private void markAllUnanalyzed(UseCase usecase) {
        if (usecase == null) return;
        for (int i = 0; i < usecase.getStepCount(); i++) {
            markAllUnanalyzed(usecase.getStep(i).getSubUseCase());
        }
        usecase.setUseCaseAnalyzed(false);
        
    }
    
    
    /**
     * Stores all usecases within this collection to a string and returns it
     */
    public String toString() {
    	StringBuffer sb = new StringBuffer();
    	sb.append(useCases.size() + " UseCases :\n");
    	
    	for(int i=0;i<useCases.size();i++) {
    		sb.append(useCases.get(i).toString() + "\n");
    	}
    	
    	return sb.toString();
    }
    
    /**
	 * Resolves step and sub-usecase relations, removes sub-usecases from usecasecollection
	 * and stores their reference to the corresponding usecase step
	 * @param markAsSubUseCases 
	 */
	public void resolveSubUseCases(boolean markAsSubUseCases) {
		UseCase usecase = null;
		UseCase subUseCase = null;	
		UseCaseStep step = null;
		String subUseCaseId = null;
		
		
		//Go through all usecases in usecasecollection
		for(int i=0;i<getUseCaseCount();i++) {
			usecase = getUseCase(i);
			
			//Then go through all steps in each usecase
			for(int j=0;j<usecase.getStepCount();j++) {
				step = usecase.getStep(j);
				
				//Check if the step has a sub-usecase id				
				subUseCaseId = step.getSubUseCaseId();
				
				if (subUseCaseId != null) {					
					//Then find a matching usecase
					subUseCase = find(subUseCaseId);
					
					//If a matching subusecase was found
					if (subUseCase != null) {
						//Set the sub usecase
						step.setSubUseCase(subUseCase);						
						
						if (markAsSubUseCases)
							subUseCase.setAsSubUseCase(true);
						
						//If the step has no name, then we get it's name from sub usecases name
						if (step.getStep().length() < 1) {
							step.setStep(subUseCase.getName());
						}						
						
					}					
				}
			}
		}
		
		//Removes all subusecase from the main list
		removeAllSubUseCase();
	}
} 
