package ucot.model;

import java.util.Set;
/**
 * ModelEditor is a interface to edit AnalyzeModel.
 * Edition is made trough it to make it possible to implement
 * undo/redo in the future.
 *
 */
public interface ModelEditor {
	/**
	 * Clears model and makes it empty.
	 */
	public void clearModel();
	
	/**
	 * @return the names of the entities that the model contains.
	 */
	public Set<String> getEntityNames();
	
	/**
	 * Adds an entity to the model.
	 * @param name The name of the entity.
	 */
	public void addEntity(String name);
	
	/**
	 * Removes entity from model.
	 * @param name The name of the entity.
	 */
	public void removeEntity(String name);
	
	/**
	 * Changes the name of the entity.
	 * @param oldName The entity name that is going to be changed.
	 * @param newName The new name for that entity.
	 */
	public void changeEntityName(String oldName, String newName);
	
	/**
	 * Does the model contain the entity?
	 * @param entityName The name of the entity which existence is checked.
	 * @return true if exists false if does not.
	 */
	public boolean containsEntity(String entityName);
	
	
	/**
	 * Adds parent to the entity.
	 * @param entityName The name of the child entity.
	 * @param parentEntity The name of the parent entity.
	 */
	public void addParent(String entityName, String parentEntity);
	
	/**
	 * Removes given parent from the entity.
	 * @param entityName The name of the child entity.
	 * @param parentEntity The name of the parent entity.
	 */
	public void removeParent(String entityName, String parentEntity);
	
	/**
	 * Removes all the parents of the entity.
	 * @param entityName the name of the entity.
	 */
	public void removeAllParents(String entityName);
	
	/**
	 * Returns the parents of the entity.
	 * @param entityName The name of the entity.
	 * @return The parents of the given entity.
	 */
	public Set<String> getParents(String entityName);
	
	/**
	 * Adds child to the entity.
	 * @param entityName The name of parent entity.
	 * @param childEntity The name of the child.
	 */
	public void addChild(String entityName, String childEntity);
	
	/**
	 * Removes given child from the entity.
	 * @param entityName The entity.
	 * @param childEntity The child to be removed.
	 */
	public void removeChild(String entityName, String childEntity);
	
	/**
	 * Removes all the children from the entity.
	 * @param entityName
	 */	
	public void removeAllChildren(String entityName);
	
	/**
	 * Returns the names of the entity's children.
	 * @param entityName The name of the entity.
	 * @return The child of the given entity.
	 */
	public Set<String> getChildren(String entityName);
	
	
	
	
	
	/**
	 * Add method to the entity.
	 * @param entityName The name of the entity.
	 * @param methodName The name of the method.
	 */
	public void addMethod(String entityName, String methodName);
	
	/**
	 * Removes method from the entity.
	 * @param entityName The entity which owns the method.
	 * @param methodName The name of the method.
	 */
	public void removeMethod(String entityName, String methodName);
	
	/**
	 * Changes method name.
	 * @param entityName The name of the entity.
	 * @param methodOldName
	 * @param methodNewName
	 */
	public void changeMethodName(String entityName, String methodOldName, String methodNewName);
	
	/**
	 * Checks if the given method exists in the entity
	 * @param entityName The name of the entity
	 * @param methodName The name of the method.
	 * @return True if the entity has given method.  
	 */
	public boolean containsMethod(String entityName, String methodName);
	
	/**
	 * Returns the names of the entity's methods.
	 * @param entityName The name of the entity
	 * @return The entity's methods' names.
	 */
	public Set<String> getMethodNames(String entityName);
	
	/**
	 * Returns set of entities that are refered by methods
	 * in the model. For example sentence:
	 * A makes B can be interpereted so that make-method of entity A referes
	 * the entity B. 
	 *  
	 * @param entityName The name of the entity that owns the method.
	 * @param methodName The methods name.
	 * @return Set of entity names that are refered by the method in some way.
	 */
	public Set<String> getEntitiesInfluencedByMethod(String entityName, String methodName);
	
	/**
	 * Adds an influence between entity's method and the given entity.
	 * 
	 * @param entityName The name of the entity that owns the method.
	 * @param methodName The methods name.
	 * @param influencedEntity
	 * 
	 * @see #getEntitiesInfluencedByMethod(String, String)
	 */
	public void addEntityInfluenceByMethod(String entityName, String methodName, String influencedEntity);
	
	
	/**
	 * Removes influnce between entity's method and the given entity.
	 * 
	 * @param entityName The name of the entity that owns the method.
	 * @param methodName The methods name.
	 * @param influencedEntity The name of the method that is influenced by the given method.
	 * @see #getEntitiesInfluencedByMethod(String, String)
	 */
	public void removeEntityInfluenceByMethod(String entityName, String methodName, String influencedEntity);
	
	
	/**
	 * Adds attribute to the entity.
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attribute.
	 */	
	public void addAttribute(String entityName, String attributeName);
	
	/**
	 * Removes attribute from the entity.
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attribute.
	 */		
	public void removeAttribute(String entityName, String attributeName);
	
	/**
	 * Checks wheter the entity contains the attribute or not.
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attribute.
	 * @return True if contains.
	 */
	public boolean containsAttribute(String entityName, String attributeName);
	
	/**
	 * Gets from part of the cardinality of the attribute relation.
	 * From part means the cardinality on the entity's side that owns the
	 * attribute.
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @return The cardinality value of the from part of the cardinal relation.
	 */
	public String getAttributeFromCardinal(String entityName, String attributeName);
	
	
	/**
	 * Gets to part of the cardinality of the attribute relation.
	 * To part means the cardinality on the attributes side.
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @return The cardinality value of the to part of the cardinal relation.
	 */
	public String getAttributeToCardinal(String entityName, String attributeName);
	
	/**
	 * Sets the cardinality on the entitys side. 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @param cardinal The cardinality value.
	 */
	public void setAttributeFromCardinal(String entityName, String attributeName, String cardinal);
	
	/**
	 * Sets the cardinality on attributes side. 
	 * @param entityName The name of the entity.
	 * @param attributeName The name of the attibute.
	 * @param cardinal The cardinality value.
	 */
	public void setAttributeToCardinal(String entityName, String attributeName, String cardinal);
	
	/**
	 * Returns the attributes of the entity. 
	 * @param entityName The name of the entity.
	 * @return The attributes.
	 */
	public Set<String> getAttributeNames(String entityName);
	
	/**
	 * Merges given AnalyzeModel to the model that this editor
	 * modifies. 
	 * This only adds things that do not yet exist in the current model.
	 * Nothing is delted or modified.
	 * 
	 * @param model The analyze model beign merged.
	 */
	public void merge(AnalyzeModel model);
	
	
	
}
