package ucot.model.relation;

import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Observable;
import java.util.Queue;
import java.util.Set;

import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;

/**
 * This is the internal representation of 
 * analyzed use case. It is build up from 
 * two basic kind of objects: items and relations
 * between them.
 * 
 * 
 * 
 * @author pajumasu
 *
 */
public class RelationalAnalyzeModel extends Observable implements AnalyzeModel {
	private Set<Item> items = new HashSet<Item>();
	private Set<Relation> relations = new HashSet<Relation>();

	ModelEditor editor;

	public RelationalAnalyzeModel(){
		editor = new RelationalAnalyzeModelEditor(this);
	}


	/**
	 * Adds item in to the model
	 * @param e Item to be added.
	 */
	protected synchronized void addItem(Item e) {
		if (items.add(e)){
			setChanged();
			notifyObservers(new Updation(UpdationType.NEW,e));
		}
	}

	/**
	 * Returns entity by its name
	 * @param name The name of the wanted entity.
	 * @return
	 */
	public Entity getEntityByName(String name) {
		Set<Item> entities = getItemsType("entity");
		//Go trough the items
		for (Item item : entities) {        	
			if (item.getName().equalsIgnoreCase(name))
				return (Entity)item;            
		}
		return null;
	}

	/**
	 * Returns spesific item by its name.
	 * @param name
	 * @return
	 */
	public Item getItemByName(String name) {
		//Go trough the items
		for (Item item : items) {

			if (item.getName().equalsIgnoreCase(name))
				return item;            
		}
		return null;
	}

	/**
	 * Returns items which type name is equal to given type name.
	 * @param typeName The type name we are searching for.
	 * @return Set if items that match the given type name.
	 */
	public Set<Item> getItemsType(String typeName) {
		Set<Item> ret = new HashSet<Item>();

		// Go trough all item
		for (Item item : items) {
			if (item.getTypeName().equals(typeName)){
				// Item type is right so return it.
				ret.add(item);                
			}
		}
		return ret;
	}

	/**
	 * Extract entities from given thing.
	 * If the thing is relation then all
	 * both to and from things are futher
	 * iteratively extracted.
	 * If the thing is item then the item 
	 * is added to this model.
	 * @param i The thing to be investigated.
	 */
	private void extractEntities(Thing i) {

		// Make queue for the items to be handled.
		Queue<Thing> tobeHandled = new LinkedList<Thing>();

		// Add the first item to the handling queue.
		tobeHandled.add(i);

		// Loop until the queue is empty.
		while (tobeHandled.size() > 0) {

			// Fetch first one from the queue.
			Thing current = tobeHandled.poll();

			if (current instanceof Item) {
				// It is simple item so add it.
				addItem((Item) current);
			} else if (current instanceof Relation) {
				// It is relation so add both ends to
				// the queue and add the relation
				// to this model.
				tobeHandled.add(((Relation) current).getFrom());
				tobeHandled.add(((Relation) current).getTo());
				relations.add((Relation) current);
			}
		}
	}

	/**
	 * Returns relations that point to given thing.
	 * @param item
	 * @return
	 */
	public Set<Relation> getRelationsTo(Thing item) {
		Set<Relation> ret = new HashSet<Relation>();

		// Go trhough all relations.
		for (Relation relation : relations) {

			if (relation.getTo().equals(item)) {
				// Given Thing is the from object of the relation
				// so return this relation.
				ret.add(relation);
			}
		}
		return ret;
	}

	/**
	 * Returns relations that originates from given thing.
	 * @param item
	 * @return
	 */
	public Set<Relation> getRelationsFrom(Thing item) {
		Set<Relation> ret = new HashSet<Relation>();

		// Go trhough all relations.
		for (Relation relation : relations) {
			if (relation.getFrom().equals(item)) {
				// Given Thing is the from object of the relation
				// so return this relation.
				ret.add(relation);
			}
		}
		return ret;
	}

	/**
	 * Returns relations that are certain type
	 * and targets given item.
	 * 
	 * getRelationsFrom(new Entity("user"), RelationType.CONTAINS);
	 * Returns all the relations that connect
	 * Things containded by the 'user' Entity. Which
	 * means methods, attributes and such.
	 * 
	 * @param item The originator.
	 * @param type The RelationType we are looking for.
	 * @return
	 */
	public Set<Relation> getRelationsTo(Thing item, RelationType type) {
		// First get relations that originates from given thing.
		Set<Relation> ret = getRelationsTo(item);

		// Next find out which satisfy the type condition.
		for (Iterator<Relation> i = ret.iterator(); i.hasNext();) {
			Relation relation = i.next();
			if (!relation.getType().equals(type)) {
				// The RelationType is wrong kind so dont return this relation
				i.remove();
			}
		}
		return ret;
	}

	/**
	 * Returns relations that are certain type
	 * and are originated from given item.
	 * 
	 * getRelationsFrom(new Entity("user"), RelationType.CONTAINS);
	 * Returns all the relations that connect
	 * Things containded by the 'user' Entity. Which
	 * means methods, attributes and such.
	 * 
	 * @param item The originator.
	 * @param type The RelationType we are looking for.
	 * @return
	 */
	public Set<Relation> getRelationsFrom(Thing item, RelationType type) {
		// First get relations that originates from given thing.
		Set<Relation> ret = getRelationsFrom(item);

		// Next find out which satisfy the type condition.
		for (Iterator<Relation> i = ret.iterator(); i.hasNext();) {
			Relation relation = i.next();
			if (!relation.getType().equals(type)) {
				// The RelationType is wrong kind so dont return this relation
				i.remove();
			}
		}
		return ret;
	}

	/**
	 * Returns relations which point to thing of spesific type
	 * and are originated from given item.
	 * 
	 * getRelationsFrom(new Entity("user"), "method");
	 * Returns all the relations that are originated from
	 * Entity called 'user' and are pointing to Thing of
	 * type 'method'. Simply this calls returns all the
	 * Relations that connects the methods of the 'user' Entity.
	 * 
	 * @param item The originator.
	 * @param toType The type of the Thing relation points.
	 * @return
	 */
	public Set<Relation> getRelationsFrom(Thing item, String toType) {
		// First get relations that originates from given thing.
		Set<Relation> ret = getRelationsFrom(item);

		// Next find out which satisfy the toType condition.
		for (Iterator<Relation> i = ret.iterator(); i.hasNext();) {
			Relation relation = i.next();
			if (!relation.getTo().getTypeName().equals(toType)) {
				// Type name is different from toType so dont return this.
				i.remove();
			}
		}
		return ret;
	}

	/**
	 * Returns relations which mark method influence and 
	 * originate from given thing. 
	 * 
	 * @param thing1 
	 * @param method
	 * @return
	 */
	public Set<Relation> getRelationsFrom(Thing thing1, Method method){
		Set<Relation> relationsFrom = getRelationsFrom(thing1);
		// Next find out which satisfy the conditions.
		for (Iterator<Relation> i = relationsFrom.iterator(); i.hasNext();) {
			Relation relation = i.next();
			if (! (relation.getName().equals(method.name) 
					&& relation.getType().equals(RelationType.INFLUENCE_BY_METHOD))) {
				i.remove();
			}
		}
		return relationsFrom;
	}

	/**
	 * Removes item from the model.
	 * @param item The item that is going to be removed
	 */
	protected synchronized void removeItem(Item item){
		Collection<Relation> from = getRelationsFrom(item);
		Collection<Relation> to = getRelationsTo(item);
		for(Relation rel : from){
			removeRelation(rel);
		}
		for(Relation rel : to){
			removeRelation(rel);
		}
		items.remove(item);
		setChanged();
		notifyObservers(new Updation(UpdationType.DELETION,item));
	}

	/*
	 * Raises this metod to accessible from
	 * this package so that model editor can
	 * use it too.
	 * 
	 * (non-Javadoc)
	 * @see java.util.Observable#setChanged()
	 */
	protected synchronized void setChanged(){
		super.setChanged();
	}

	/**
	 * Removes given relation.
	 * @param relation Relation to be removed.
	 */
	protected synchronized void removeRelation(Relation relation){
		relations.remove(relation);
		setChanged();
		notifyObservers(new Updation(UpdationType.DELETION, relation));
	}


	/**
	 * Adds relation to this model.
	 * @param relation The relation beign added.
	 */
	protected synchronized void addRelation(Relation relation) {
		// Extract (and add) items and sub relations from the relation.
		extractEntities(relation);

		// Add the relation itself to the model.
		relations.add(relation);
		setChanged();
		notifyObservers(new Updation(UpdationType.NEW,relation));
	}



	// --------------------- AnalyzeModel Interface

	/* (non-Javadoc)
	 * @see ucot.model.AnalyzeModel#getEditor()
	 */
	public ModelEditor getEditor(){
		return editor;
	}


	/* (non-Javadoc)
	 * @see ucot.model.AnalyzeModel#clear()
	 */
	public synchronized void clear(){
		items.clear();
		relations.clear();
		setChanged();
		notifyObservers(new Updation(UpdationType.CLEAR, null));
	}


	/* (non-Javadoc)
	 * @see ucot.model.AnalyzeModel#readySignal()
	 */
	public void readySignal() {
		setChanged();
		notifyObservers(new Updation(UpdationType.READY, null));
	}


	/* (non-Javadoc)
	 * @see ucot.model.AnalyzeModel#updationStartedSignal()
	 */
	public void updationStartedSignal() {
		setChanged();
		notifyObservers(new Updation(UpdationType.MODIFICATION_STARTED, null));	
	}
}
