package ucot.model.relation;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import sun.reflect.ReflectionFactory.GetReflectionFactoryAction;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;

public class RelationalAnalyzeModelEditor implements ModelEditor {
	
	Logger logger = Logger.getLogger("ucot.model");
	
	private RelationalAnalyzeModel model;
	
	public RelationalAnalyzeModelEditor(RelationalAnalyzeModel model) {
		this.model = model;
	}
	
	public void addMethod(String entityName, String methodName) {
		Entity entity = model.getEntityByName(entityName);
		if (entity == null){
			throw new Error("No such entity.");
		}
		addMethod(entity,methodName);
	}
	
	private void addMethod(Entity entity, String methodName) {
		if (entity == null){
			throw new Error("Entity is null.");
		}
		if (! containsMethod(entity, methodName)){
			model.addRelation(new Relation(entity, RelationType.CONTAINS, new Method(methodName)));
			logger.finest("Method '"+methodName+"' added to entity '"+entity.getName()+"'");
		}
	}
	
	public void addEntity(String name) {
		Entity entity = model.getEntityByName(name);
		if (null == entity ){
			// If not already there then create and add.
			entity = new Entity(name);
			model.addItem(entity);
			logger.finest("Entity '"+name+"' added.");
		} 
		
	}
	
	public boolean containsMethod(String objectName, String methodName) {
		Entity entity = model.getEntityByName(objectName);
		return containsMethod(entity, methodName);
	}
	
	private boolean containsMethod(Entity entity, String methodName) {
		if (entity == null) throw new Error("Entity is null.");
		Set<Relation> methodRels = model.getRelationsFrom(entity,Method.TYPE_NAME);
		for(Relation relation : methodRels){
			if (relation.getTo().getName().equals(methodName)) return true;
		}
		return false; 
	}
	
	public boolean containsEntity(String objectName) {
		Entity entity = model.getEntityByName(objectName);
		return entity != null;
	}
	
	public void changeMethodName(String objectName, String methodOldName,
			String methodNewName) {
	}
	
	public void changeEntityName(String oldName, String newName) {
		Entity entity = model.getEntityByName(oldName);
		entity.setName(newName);
		model.setChanged();
		model.notifyObservers(new Updation(UpdationType.MODIFY, entity));
	}
	
	public void removeMethod(String objectName, String methodName) {
		Entity entity = model.getEntityByName(objectName);
		if (entity == null){
			throw new Error("No such entity.");
		}
		// Remove method.
		Set<Relation> relations = model.getRelationsFrom(entity, Method.TYPE_NAME);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(methodName)){
				model.removeRelation(relation);
			}
		}		
		
		// Remove influences by method.
		relations = model.getRelationsFrom(entity, Entity.TYPE_NAME);
		for(Relation relation : relations){
			if (relation.type.equals(RelationType.INFLUENCE_BY_METHOD) && relation.getName().equals(methodName)){
				model.removeRelation(relation);
			}
		}		
		
		
	}
	
	public void removeEntity(String name) {
		Entity entity = model.getEntityByName(name);
		if ( entity == null ) throw new Error("No such entity (" + name + ")");
		model.removeItem(entity);
	}
	
	public void addAttribute(String entityName, String attributeName) {
		Entity entity = model.getEntityByName(entityName);
		Entity attribute = model.getEntityByName(attributeName);
		if (entity == null){
			throw new Error("Entity '"+entityName+"' does not exist.");
		}
		if ( attribute == null ) {
			throw new Error("Attribute '"+attributeName+"' does not exist.");
		}
		if (! containsAttribute(entityName, attributeName)){
			model.addRelation(new Relation(entity, RelationType.CONTAINS, attribute));
			logger.finest("Attribute '"+attributeName+"' added to entity '"+entityName+"'");
		}
	}
	
	public void addAttribute(Entity entity, String attributeName) {
		Entity attribute = model.getEntityByName(attributeName);
		if (entity == null){
			throw new Error("Entity is null does not exist.");
		}
		if ( attribute == null ) {
			throw new Error("Attribute '"+attributeName+"' does not exist.");
		}
		if (! containsAttribute(entity, attributeName)){
			model.addRelation(new Relation(entity, RelationType.CONTAINS, attribute));
			logger.finest("Attribute '"+attributeName+"' added to entity '"+entity.getName()+"'");
		}
	}
	
	public void removeAttribute(String entityName, String attributeName) {
		Entity entity = model.getEntityByName(entityName);
		Entity attribute = model.getEntityByName(attributeName);
		if (entity == null){
			throw new Error("Entity '"+entityName+"' does not exist.");
		}
		if ( attribute == null ) {
			throw new Error("Attribute '"+attributeName+"' does not exist.");
		}
		// Remove attribute.
		Set<Relation> relations = model.getRelationsFrom(entity, RelationType.CONTAINS);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(attributeName) &&
					relation.getTo() instanceof Entity ){
				model.removeRelation(relation);
			}
		}
	}
	
	public boolean containsAttribute(String entityName, String attributeName) {
		Entity entity = model.getEntityByName(entityName);
		return containsAttribute(entity, attributeName);
	}
	
	public boolean containsAttribute(Entity entity, String attributeName) {
		Set<Relation> relations = model.getRelationsFrom(entity, RelationType.CONTAINS);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(attributeName) &&
					relation.getTo() instanceof Entity) return true;
		}
		return false;
	}
	
	public Set<String> getAttributeNames(String entityName) {
		Entity entity = model.getEntityByName(entityName);
		return getAttributeNames(entity);
	}
	
	public Set<String> getAttributeNames(Entity entity) {
		Set<Relation> methodRels = model.getRelationsFrom(entity, RelationType.CONTAINS);
		Set<String> ret = new HashSet<String>();
		for(Relation relation : methodRels){
			if ( relation.getTo() instanceof Entity )
				ret.add(relation.getTo().getName());
		}
		return ret; 
	}
	
	public void clearModel() {
		model.clear();
	}
	
	public Set<String> getMethodNames(String objectName) {
		Entity entity = model.getEntityByName(objectName);
		Set<Relation> methodRels = model.getRelationsFrom(entity,Method.TYPE_NAME);
		Set<String> ret = new HashSet<String>();
		for(Relation relation : methodRels){
			ret.add(relation.getTo().getName());
		}
		return ret; 
	}
	
	public Set<String> getEntityNames() {
		Set<Item> ents = model.getItemsType("entity");
		Set<String> ret = new HashSet<String>();
		for(Item item : ents){
			ret.add(item.getName());
		}
		return ret; 
	}
	
	public boolean containsRelation(String fromEntity, String toEntity) {
		Entity from = model.getEntityByName(fromEntity); 
		Entity to = model.getEntityByName(toEntity);
		
		if (null == from || null == to) return false;
		
		Set<Relation> relations = model.getRelationsFrom(from);
		for(Relation relation : relations){
			if (relation.getTo().equals(to)) return true;
		}
		return false;
	}
	
	public Set<String> getEntitiesInfluencedByMethod(String entityName, String methodName) {
		Entity entity = model.getEntityByName(entityName);
		return getEntitiesInfluencedByMethod(entity, methodName);
	}
	
	private Set<String> getEntitiesInfluencedByMethod(Entity entity, String methodName) {
		Set<String> ret = new HashSet<String>();
		if (entity == null) return ret;
		Set<Relation> methodRels = model.getRelationsFrom(entity, RelationType.INFLUENCE_BY_METHOD);		
		for(Relation methodRelation : methodRels){
			if (methodRelation.getName().equals(methodName))
				ret.add(methodRelation.getTo().getName());
		}
		return ret; 
	}
	
	public void addEntityInfluenceByMethod(String entityName, String methodName, String influencedEntity) {
		addMethod(entityName, methodName);
		Entity fromEntity = model.getEntityByName(entityName);
		Entity toEntity = model.getEntityByName(influencedEntity);
		Relation relation = new Relation(fromEntity, methodName, toEntity);        
		model.addRelation(relation);
	}
	
	public void addEntityInfluenceByMethod(Entity fromEntity, String methodName, String influencedEntity) {
		addMethod(fromEntity, methodName);
		Entity toEntity = model.getEntityByName(influencedEntity);
		Relation relation = new Relation(fromEntity, methodName, toEntity);        
		model.addRelation(relation);
	}
	
	public void removeEntityInfluenceByMethod(String entityName, String methodName, String influencedEntity) {
		Entity fromEntity = model.getEntityByName(entityName);
		Entity toEntity = model.getEntityByName(influencedEntity);
		Set<Relation> relations = model.getRelationsFrom(fromEntity, Entity.TYPE_NAME);
		for(Relation relation : relations){
			if (relation.getName().equals(methodName) && relation.type.equals(RelationType.INFLUENCE_BY_METHOD) && relation.getTo().equals(toEntity)){
				model.removeRelation(relation);
			}
		}		
	}
	
	public String getAttributeFromCardinal(String entityName, String attributeName) {
		Entity entity = model.getEntityByName(entityName);
		Set<Relation> relations = model.getRelationsFrom(entity, RelationType.CONTAINS);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(attributeName) &&
					relation.getTo() instanceof Entity){
				return relation.getFromCardinal();
			}
		}
		throw new Error("No such attribute '"+attributeName+"' in entity '"+entityName+"'.");
	}
	
	
	public String getAttributeToCardinal(String entityName, String attributeName) {
		Entity entity = model.getEntityByName(entityName);
		Set<Relation> relations = model.getRelationsFrom(entity, RelationType.CONTAINS);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(attributeName) &&
					relation.getTo() instanceof Entity){
				return relation.getToCardinal();
			}
		}
		throw new Error("No such attribute '"+attributeName+"' in entity '"+entityName+"'.");
	}
	
	public void setAttributeFromCardinal(String entityName, String attributeName, String cardinal) {
		Entity entity = model.getEntityByName(entityName);
		setAttributeFromCardinal(entity, attributeName, cardinal);
	}
	
	public void setAttributeFromCardinal(Entity entity, String attributeName, String cardinal) {
		Set<Relation> relations = model.getRelationsFrom(entity, RelationType.CONTAINS);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(attributeName) &&
					relation.getTo() instanceof Entity){
				relation.setFromCardinal(cardinal);
				model.readySignal();
				return;
			}
		}
		throw new Error("No such attribute '"+attributeName+"' in entity '"+entity.getName()+"'.");
		
	}
	
	
	public void setAttributeToCardinal(String entityName, String attributeName, String cardinal) {
		Entity entity = model.getEntityByName(entityName);
		setAttributeToCardinal(entity, attributeName, cardinal);
	}
	
	public void setAttributeToCardinal(Entity entity, String attributeName, String cardinal) {
		Set<Relation> relations = model.getRelationsFrom(entity, RelationType.CONTAINS);
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(attributeName) &&
					relation.getTo() instanceof Entity){
				relation.setToCardinal(cardinal);
				model.readySignal();
				return;
			}
		}
		throw new Error("No such attribute '"+attributeName+"' in entity '"+entity.getName()+"'.");
	}

	public void merge(AnalyzeModel fromModel) {
		model.updationStartedSignal();
		ModelEditor fromEditor = fromModel.getEditor();
		Set<String> fromEntities = fromEditor.getEntityNames();
		Set<String> currentEntities = this.getEntityNames();
		
		Set<String> newEntities = new HashSet<String>();
		newEntities.addAll(fromEntities);
		newEntities.removeAll(currentEntities);

		Set<String> missingEntities = new HashSet<String>();
		missingEntities.addAll(currentEntities);
		missingEntities.removeAll(fromEntities);

		
		Set<String> possiblyChangedEntities = new HashSet<String>();
		possiblyChangedEntities.addAll(fromEntities);
		possiblyChangedEntities.removeAll(missingEntities);
		
		// Create new entities.
		for(String name : newEntities){
			Entity newEntity = new Entity(name);
			model.addItem(newEntity);
		}
		
		// Update possibly changed.
		for(String entityName : possiblyChangedEntities){
			Entity entity = (Entity) model.getEntityByName(entityName);
			
			// Methods
			Set<String> fromMethods = fromEditor.getMethodNames(entityName);
			Set<String> currentMethods = this.getMethodNames(entityName);
			
			Set<String> newMethods = new HashSet<String>();
			newMethods.addAll(fromMethods);
			newMethods.removeAll(currentMethods);

			Set<String> missingMethods = new HashSet<String>();
			newMethods.addAll(currentMethods);
			newMethods.removeAll(fromMethods);

			Set<String> possiblyChangedMethods = new HashSet<String>();
			possiblyChangedMethods.addAll(fromMethods);
			possiblyChangedMethods.removeAll(missingMethods);

			// Add new methods.
			for(String method : newMethods){
				this.addMethod(entity, method);
			}
			
			// Check method influences
			for(String method : possiblyChangedMethods){
				Set<String> fromInfluenced = fromEditor.getEntitiesInfluencedByMethod(entityName, method);
				Set<String> currentInfluenced = this.getEntitiesInfluencedByMethod(entity,method);
				
				Set<String> newInfluences = new HashSet<String>();
				newInfluences.addAll(fromInfluenced);
				newInfluences.removeAll(currentInfluenced);
				
				for(String influence : newInfluences){
					addEntityInfluenceByMethod(entity,method,influence);					
				}				
			}


			// Attributes
			Set<String> fromAttributes = fromEditor.getAttributeNames(entityName);
			Set<String> currentAttributes = this.getAttributeNames(entity);
			
			Set<String> newAttributes  = new HashSet<String>();
			newAttributes.addAll(fromAttributes);
			newAttributes.removeAll(currentAttributes);
			
			for(String attributeName : newAttributes){
				String fromCardinal = fromEditor.getAttributeFromCardinal(entityName,attributeName);
				String toCardinal = fromEditor.getAttributeToCardinal(entityName,attributeName);
				
				this.addAttribute(entity,attributeName);
				this.setAttributeFromCardinal(entity, attributeName, fromCardinal);
				this.setAttributeToCardinal(entity, attributeName, toCardinal);

				
			}	
		}
		model.readySignal();
	}

	public void addChild(String entityName, String childName) {
		addParent(childName, entityName);
	}

	public void addParent(String entityName, String parentName) {
		Entity entity = model.getEntityByName(entityName);
		if (null == entity ){
			throw new Error("No such entity.'" + entityName + "'.");
		}
		Entity parentEntity = model.getEntityByName(parentName);
		if (null == parentEntity ){
			throw new Error("No such entity '" + parentName + "'.");
		}
		if (! this.getParents(entityName).contains(parentName)){
			Relation relation = new Relation(entity,RelationType.INHERITS,parentEntity);		
			model.addRelation(relation);
		}
	}

	public Set<String> getChildren(String entityName) {
		Entity entity = model.getEntityByName(entityName);
		Set<String> ret = new HashSet<String>(); 
		if (null == entity ){
			throw new Error("No such entity.'" + entityName + "'.");
		}
		Set<Relation> relations = model.getRelationsTo(entity,RelationType.INHERITS);
		for(Relation relation : relations){
			ret.add(relation.getFrom().getName());
		}
		return ret;
	}

	public Set<String> getParents(String entityName) {
		Entity entity = model.getEntityByName(entityName);
		Set<String> ret = new HashSet<String>(); 
		if (null == entity ){
			return ret;
			//throw new Error("No such entity.'" + entityName + "'.");
		}
		Set<Relation> relations = model.getRelationsFrom(entity,RelationType.INHERITS);
		for(Relation relation : relations){
			ret.add(relation.getTo().getName());
		}
		return ret;
	}

	public void removeChild(String entityName, String childEntity) {
		removeParent(childEntity,entityName);		
	}

	public void removeAllChildren(String entityName) {
		Set<String> children = getChildren(entityName);
		for(String child :children){
			removeChild(entityName,child);
		}
		
	}

	public void removeAllParents(String entityName) {
		Set<String> parents = getParents(entityName);
		for(String parent : parents){
			removeParent(entityName,parent);
		}
	}

	public void removeParent(String entityName, String parentName) {
		Entity entity = model.getEntityByName(entityName);
		if (null == entity ){
			throw new Error("No such entity.'" + entityName + "'.");
		}
		Entity parentEntity = model.getEntityByName(parentName);
		if (null == parentEntity ){
			throw new Error("No such entity '" + parentName + "'.");
		}
		Set<Relation> relations = model.getRelationsFrom(entity,RelationType.INHERITS);		
		for(Relation relation : relations){
			if (relation.getTo().getName().equals(parentName)){
				model.removeRelation(relation);
				return;
			}
		}
	}
}

