package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ucot.core.ControlInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.parser.ParserInterface;

/**
 * Dialog to ask from user wich parser and heuristic he/she wants to use.
 * 
 * @author ilanliuk
 *
 */
public class AddToModelWithDialog extends JDialog {
    
    private static ParserInterface selectedParser = null;
    private static HeuristicInterface selectedHeuristic = null;
    
    private JLabel parserLabel = new JLabel(Messages.getString("AddToModelWithDialog.SELECT_PARSER")); // 
    private JComboBox parserComboBox = new JComboBox();
    private JLabel heuristicLabel = new JLabel(Messages.getString("AddToModelWithDialog.SELECT_HEURISTIC")); //$NON-NLS-1$
    private JComboBox heuristicComboBox = new JComboBox();

    
    private ControlInterface core;
        
    /**
     * Default constructor for dialog.
     * @param owner Frame
     * @param core ControlInterface
     * @throws HeadlessException 
     */
    public AddToModelWithDialog(Frame owner, ControlInterface core) throws HeadlessException {
        super(owner, Messages.getString("AddToModelWithDialog.ADD_TO_MODEL_WITH"), true);
        
        this.core = core;
        
        this.setLayout(new BorderLayout());

        JPanel parserPanel = new JPanel();
        parserPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
        parserPanel.add(parserLabel);
        parserPanel.add(parserComboBox);
        
        JPanel heuristicPanel = new JPanel();
        heuristicPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 10));
        heuristicPanel.add(heuristicLabel);
        heuristicPanel.add(heuristicComboBox);

        // set up ComboBoxes
        InitializeComboBoxes();
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        
        JButton okButton = new JButton(Messages.getString("AddToModelWithDialog.OK_BUTTON_CAPTION"));
        // Create actionlistener for okButton
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Object o = parserComboBox.getSelectedItem();
                
                selectedParser = (ParserInterface) o;
                
                o = heuristicComboBox.getSelectedItem();
                selectedHeuristic = (HeuristicInterface) o;
                
                setVisible(false);
            }
        });
        buttonPanel.add(okButton);
        
        JButton cancelButton = new JButton(Messages.getString("AddToModelWithDialog.CANCEL_BUTTON_CAPTION")); 
        // Create actionlistener for cancelButton
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                
                // set selected parser and heuristic null
                // 
                selectedHeuristic = null;
                selectedParser = null;
                
                setVisible(false);
            }
        });
        buttonPanel.add(cancelButton);

        this.getContentPane().add(parserPanel, BorderLayout.NORTH);
        this.getContentPane().add(heuristicPanel, BorderLayout.CENTER);
        this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);
        
        this.setResizable(false);
        this.pack();
    }

    /**
     *  Sets up the comboboxes(clears them and adds new items)
     */
    private void InitializeComboBoxes() {
        
        parserComboBox.removeAllItems();
        heuristicComboBox.removeAllItems();
        
        // set up parserComboBox
        List<ParserInterface> parsers = core.getParsers();
        for (ParserInterface parser : parsers) {
            parserComboBox.addItem(parser);
        }        
        // set default parser as default selection of combobox
        parserComboBox.setSelectedItem(core.getCurrentParser());
        
        // set up heuristicComboBox
        List<HeuristicInterface> heuristics = core.getHeuristics();
        
        for (HeuristicInterface heuristic : heuristics) {
            heuristicComboBox.addItem(heuristic);
        }
        // set default heuristic as default selection of combobox
        heuristicComboBox.setSelectedItem(core.getCurrentHeuristic());
        
    }
    

    /**
     * Static method to create and show dialog. Returns true if user
     * clicked OK button in dialog else returns false.
     * @param core ControlInterface
     * @return true if user clicked OK in dialog, else false
     */    
    public static boolean showDialog(ControlInterface core) {
        
        // FIXME if user closes dialog from the red cross this should
        // return false. Now it returns true because selectedHeuristic
        // and selectedParser are not set null.
        
        // create new dialog for selecting parser and heuristic to use
        AddToModelWithDialog dialog = new AddToModelWithDialog(null, core);

        // show dialog
        dialog.setVisible(true);
        
        // if parser and heuristic are selected (user pressed OK)
        // return true if not return false
        if (selectedHeuristic != null && selectedParser != null) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns heuristic that user selected, or null if no
     * heuristic was selected.
     * @return HeuristicInterface
     */
    public static HeuristicInterface getHeuristic() {
        return selectedHeuristic;
    }

    /**
     * Returns parser that user selected, or null if no
     * parser was selected.
     * @return ParserInterface
     */
    public static ParserInterface getParser() {
        return selectedParser;
    }
    
}
