package ucot.ui.gui;

import java.awt.Color;

/**
 * This class implements a color theme for dot markup language,
 * which is useful for DotPanel when figuring out color strings.
 * It is easy to change the color theme using the method
 * 'changeToColorTheme' and giving it the identifier of the
 * preferred color theme.
 * 
 * All themes available currently have to be hard coded here
 * because of the way how dot understands colors.
 * 
 * TODO Make color themes more dynamic?
 * TODO Add more color themes.
 * 
 * @author tujupien
 */
public class DotColorTheme {
	private interface CallBack {
    	public void changeToDefaultColorTheme();
    }
	
    private String highlightFillColor;
    private String highlightFontColor;
    private String highlightEdgeColor;
    private String normalFillColor;
    private String normalFontColor;
    private String normalEdgeColor;
    private String backgroundColor;
    
    // Define an identifier for all color themes here.
    public final static int BLACK_ON_WHITE_WITH_BLUE_HIGHLIGHTS = 1;
    public final static int BLACK_ON_WHITE_WITH_GREEN_HIGHLIGHTS = 2;
    // TODO New color IDs here.
    
    // Initialize default color theme initializer.
    private final static int DEFAULT_COLOR_THEME = BLACK_ON_WHITE_WITH_BLUE_HIGHLIGHTS;
	private final CallBack defaultColorTheme = new CallBack() {
		public void changeToDefaultColorTheme() {
			changeToBlackOnWhiteWithBlueHighlights();
		}
	};
    
	/**
	 * Constructor for DotColorTheme class.
	 * @param theme Initial color theme identifier.
	 */
    public DotColorTheme(int theme) {
    	changeToColorTheme(theme);
    }
    
    /**
     * Default constructor for DotColorTheme class which initially
     * uses the default color theme.
     */
    public DotColorTheme() {
    	this(DEFAULT_COLOR_THEME);
    }
    
    /**
     * Change color theme to black text on white background
     * with blue highlight colors.
     */
    private void changeToBlackOnWhiteWithBlueHighlights() {
    	highlightFillColor = "lightblue";
        highlightFontColor = "dodgerblue";
        highlightEdgeColor = "dodgerblue";
        normalFillColor = "white";
        normalFontColor = "black";
        normalEdgeColor = "black";
        backgroundColor = "white";
    }
    
    /**
     * Change color theme to black text on white background
     * with green highlight colors.
     */
    private void changeToBlackOnWhiteWithGreenHighlights() {
    	highlightFillColor = "chartreuse";
        highlightFontColor = "forestgreen";
        highlightEdgeColor = "forestgreen";
        normalFillColor = "white";
        normalFontColor = "black";
        normalEdgeColor = "black";
        backgroundColor = "white";
    }
    
    // TODO New color changers here.
    
    /**
     * Method for changing the color theme. Default theme
     * is used if the given theme identifier is invalid.
     * @param newTheme Identifier of the new theme.
     */
    public void changeToColorTheme(int newTheme) {
    	switch (newTheme) {
			case BLACK_ON_WHITE_WITH_BLUE_HIGHLIGHTS:
				changeToBlackOnWhiteWithBlueHighlights();
				break;
			case BLACK_ON_WHITE_WITH_GREEN_HIGHLIGHTS:
				changeToBlackOnWhiteWithGreenHighlights();
				break;
		    // TODO New color changer callers here.
			default:
				defaultColorTheme.changeToDefaultColorTheme();
				break;
		}
    }
    
    /**
     * Method which formats highlighted item's color attributes
     * into dot's syntax.
     * @return Highlight nodes' or edges' string in dot's syntax.
     */
    public String getHighlightedColorString() {
    	return "style=\"filled" +
    		"\", color=\"" + highlightEdgeColor +
    	    "\", fillcolor=\"" + highlightFillColor +
    	    "\", fontcolor=\"" + highlightFontColor + "\"";
    }
    
    /**
     * Method for getting the background color of the whole graph
     * in dot's syntax.
     * @return Background color string in dot's syntax.
     */
    public String getBackgroundColorString() {
    	return "bgcolor=\"" + backgroundColor + "\"";
    }

    /**
     * Method which formats normal item's color attributes
     * into dot's syntax.
     * @return Normal nodes' or edges' string in dot's syntax.
     */
    public String getNormalColorString() {
    	return "style=\"filled" +
		"\", color=\"" + normalEdgeColor +
	    "\", fillcolor=\"" + normalFillColor +
	    "\", fontcolor=\"" + normalFontColor + "\"";
    }
    
    /**
     * Method for getting the appropriate color string for
     * an entity based on its highlight status.
     * @param isHighlighted Defines wether or not the returned
     *   color string is supposed to be for an highlighted entity
     *   or a normal entity.
     * @return Returns appropriate color string in dot's syntax.
     */
    public String getColorString(boolean isHighlighted) {
    	if ( isHighlighted ) return getHighlightedColorString();
    	return getNormalColorString();
    }
    
    /**
     * Method for getting the background color of the graph as
     * a java object. This helps to figure out the background
     * color of the DotPanel.
     * @return Graph background color as a Java object.
     */
    public Color getBackgroundColorAsJavaObject() {
    	// TODO Complete this dot color conversion map.
    	if ( backgroundColor.equalsIgnoreCase("white") ) return Color.WHITE;
    	if ( backgroundColor.equalsIgnoreCase("black") ) return Color.BLACK;
    	if ( backgroundColor.equalsIgnoreCase("red") ) return Color.RED;
    	if ( backgroundColor.equalsIgnoreCase("green") ) return Color.GREEN;
    	if ( backgroundColor.equalsIgnoreCase("blue") ) return Color.BLUE;
    	
    	// Set default background color here in case the following
    	// dot color map is not complete.
    	return Color.WHITE;
    }
}
