package ucot.ui.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.model.Updation;
import ucot.model.Updation.UpdationType;

/**
 * A dot panel class inherited from JPanel class which is used for
 * drawing dot graphs from the given analyze model. This class
 * implements the observer interface which allows objects to keep
 * track on the current status of this panel.
 * 
 * This panel also observes the given analyze model and updates
 * the graph when the model changes.
 * 
 * There is also a method which allows highlighting any submodels
 * from the given analyze model.
 * 
 * @author ilanliuk, tujupien.
 */
public class DotPanel extends JPanel implements Observer {

    public enum ExportImageType {PNG};
    
    // Temporary files used by dot and path to the dot executable.
    protected static String TEMP_INPUT_FILE = "temp.dot";
    protected static String TEMP_OUTPUT_FILE = "graph.png";
    protected static String DOT_PATH = "./dot/dot.exe";

    private AnalyzeModel analyzeModel;
    private static DotColorTheme colorTheme = new DotColorTheme();
    
    // Sets for highlighting and highlight mapping.
    private Set<String> entitiesToBeHighlighted = null;
    private Set<String> entitiesInCurrentModel = null;
	private Set<String> entitiesRequestedForHighlighting = null;
    private Set<Vector<String>> parentsToBeHighlighted = null;
    private Set<Vector<String>> parentsInCurrentModel = null;
	private Set<Vector<String>> parentsRequestedForHighlighting = null;
    private Set<Vector<String>> methodsToBeHighlighted = null;
    private Set<Vector<String>> methodsInCurrentModel = null;
	private Set<Vector<String>> methodsRequestedForHighlighting = null;
    private Set<Vector<String>> attributesToBeHighlighted = null;
    private Set<Vector<String>> attributesInCurrentModel = null;
	private Set<Vector<String>> attributesRequestedForHighlighting = null;
	
    private BufferedImage modelImage = null;

    private boolean isUpdating;
    
    /**
     * Constructor for DotPanel. This constructor adds an observer
     * to the analyze model and sets its own status to updating.
     * @param analyzeModel analyzeModel to use with this DotPanel.
     */
    public DotPanel(AnalyzeModel analyzeModel) {
        this.analyzeModel = analyzeModel;
        analyzeModel.addObserver(this);
        this.isUpdating = true;
    }

    /* (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable observableObject, Object updationArg) {
        if ( !(updationArg instanceof Updation) )
            return;
        
        // Change updation status.
        Updation updation = (Updation) updationArg;
        if ( updation.getType() == UpdationType.MODIFICATION_STARTED ) {
            this.isUpdating = false;
        }
        if ( updation.getType() == UpdationType.READY ) {
            this.isUpdating = true;
        }
        
        // Update graph if required.
        if ( this.isUpdating ) {
            this.updateModel(this.analyzeModel);
        }
    }

    /**
     * Saves current image to given file in given format.
     * @param target Target file.
     * @param imageFormat Image format to use.
     */
    public void exportImage(File target, ExportImageType imageFormat) {
        
        if ( imageFormat == ExportImageType.PNG ) {
            try {
                ImageIO.write(modelImage, "png", target);
            } catch (IOException ex){
                ex.printStackTrace();
                throw new Error("Exporting image failed!");
            }
        } 
    }
    
    /**
     * This method runs the dot executable with the given dot input file
     * and returns the location of the image file.
     * @param dotInputFile Input file for dot executable.
     * @return File where the image is located. Null is returned if
     *   something went wrog.
     */
    private File runDot(File dotInputFile) {

        // Check if we can find the dot executable.
        File dotExecutable = new File(DOT_PATH);
        if (!dotExecutable.exists()) {
            System.err.println("Error in DotPanel.java: can't find the dot.exe in given path '"
                            + dotExecutable.getAbsolutePath() + "'");
            return null;
        }

        File outputFile = new File(TEMP_OUTPUT_FILE);

        // TODO Debug this on Linux.
        // Get the output from dot executable as a PNG image.
        String executeCommand = dotExecutable.getAbsolutePath() + " -Tpng "
                + dotInputFile.getAbsolutePath() + " -o "
                + outputFile.getAbsolutePath();

        // Run dot executable.
        Process dotProcess = null;
        try {
            dotProcess = Runtime.getRuntime().exec(executeCommand);
        } catch (IOException e) {
            System.err.println(e.getMessage());
            outputFile = null;
        }

        if (dotProcess == null)
            return null;

        try {
            dotProcess.waitFor();
        } catch (InterruptedException e) {
            System.err.println(e.getMessage());
        }

        return outputFile;
    }
    
    /**
     * Method for writing the methods of one entity to the dot file.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose methods are being written.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityMethods(BufferedWriter writer, String entity) throws IOException {
    	ModelEditor editor = analyzeModel.getEditor();
    	Set<String> methods = editor.getMethodNames(entity);
        
        try {
	        for ( String method : methods ) {
	            Set<String> targets = editor.getEntitiesInfluencedByMethod(
	                    entity, method);
	
	            for ( String target : targets ) {
	                writer.write("  \"" + entity + "\" -> \"" + target
	                        + "\" [label=\"" + method
	                        + "\", arrowhead=\"vee\", ");
	                Vector<String> compare = new Vector<String>();
	                compare.add(entity);
	                compare.add(method);
	                compare.add(target);
	                writer.write(colorTheme.getColorString(methodsToBeHighlighted != null &&
	            			methodsToBeHighlighted.contains(compare)));
	                writer.write("];");
	                writer.newLine();
	            }
	        }
        } catch (IOException e) {
        	throw e;
        }
    }
    
    /**
     * Method for writing the attributes of one entity to the dot file.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose attributes are being written.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityAttributes(BufferedWriter writer, String entity) throws IOException {
    	ModelEditor editor = analyzeModel.getEditor();
        Set<String> attributes = editor.getAttributeNames(entity);
        for ( String attribute : attributes ) {
        	writer.write("  \"" + entity + "\" -> \"" + attribute +
        			"\" [taillabel=\""+editor.getAttributeFromCardinal(entity, attribute)
        			+"\", headlabel=\""+editor.getAttributeToCardinal(entity, attribute)+
        			"\", arrowtail=\"diamond\", arrowsize=\"1.3\", labeldistance=\"1.7\""+
        			", arrowhead=\"none\", ");
        	Vector<String> compare = new Vector<String>();
        	compare.add(entity);
        	compare.add(attribute);
        	writer.write(colorTheme.getColorString(attributesToBeHighlighted != null &&
        			attributesToBeHighlighted.contains(compare)));
        	writer.write("];");
        	writer.newLine();
        }
    }
    
    /**
     * Method for writing the parents of one entity to the dot file.
     * @param writer BufferedWriter to use for writing.
     * @param entity Name of the entity whose parents are being written.
     * @throws IOException If something goes wrong an exception is thrown.
     */
    private void writeEntityParents(BufferedWriter writer, String entity) throws IOException {
    	Set<String> parents = analyzeModel.getEditor().getParents(entity);
        for ( String parent : parents ) {
        	writer.write("  \"" + entity + "\" -> \"" + parent + 
        			"\" [arrowhead=\"onormal\", arrowsize=\"1.4\", ");
        	Vector<String> compare = new Vector<String>();
        	compare.add(entity);
        	compare.add(parent);
        	writer.write(colorTheme.getColorString(parentsToBeHighlighted != null &&
        			parentsToBeHighlighted.contains(compare)));
        	writer.write("];");
        	writer.newLine();
        }
    }
    
    /**
     * Method for writing all entities to the dot file with all their
     * attribute, method and parent relationships.
     * @param writer BufferedWriter to use for writing.
     * @throws IOException An exception is thrown if something goes wrong.
     */
    private void writeEntities(BufferedWriter writer) throws IOException {
        Set<String> entityNames = analyzeModel.getEditor().getEntityNames();
        
        try {
	    	for ( String entity : entityNames ) {
	        	
	            writer.write("  \"" + entity + "\" [");
	            writer.write(colorTheme.getColorString(entitiesToBeHighlighted != null &&
	        			entitiesToBeHighlighted.contains(entity)));
	            writer.write("];");
	            writer.newLine();
	            
	            writeEntityMethods(writer, entity);
	            writeEntityAttributes(writer, entity);
	            writeEntityParents(writer, entity);
	        }
        } catch (IOException e) {
        	throw e;
        }
    }
    
    /**
     * Writes the analyze model to an external file in dot syntax.
     * @return File containing the analyze model in dot syntax.
     *   Null is returned if something went wrong.
     */
    private File createDotFile() {

        File result = new File(TEMP_INPUT_FILE);

        // Initialize stream to file for writing.
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(result));

            // Write starting tag into file and graph-wide attributes.
            writer.write("digraph G { [" + colorTheme.getBackgroundColorString() + "]");
            writer.newLine();

            // Write the entities and their relationships to the dot file.
            writeEntities(writer);

            // Write end tag into file.
            writer.write("}");
            writer.newLine();

            writer.flush();
            writer.close();

        } catch (IOException e) {
            System.err.println(e.getMessage());
            return null;
        }

        return result;
    }
    
    /**
     * Method for mapping the current analyze model. This is required to
     * figure out differences with the submodel that is required to be
     * highlighted.
     */
    private void mapCurrentModel() {
    	for ( String entity : entitiesInCurrentModel ) {
    		for ( String method : analyzeModel.getEditor().getMethodNames(entity) ) {
    			for ( String target : analyzeModel.getEditor().
    					getEntitiesInfluencedByMethod(entity, method) ) {
    				// Method vector stores the method name plus its owner and
    				// target entities.
    				Vector<String> methodVector = new Vector<String>();
    				methodVector.add(entity);
    				methodVector.add(method);
    				methodVector.add(target);
    				methodsInCurrentModel.add(methodVector);
    			}
    		}
    		
    		for ( String attribute : analyzeModel.getEditor().getAttributeNames(entity) ) {
    			// Attribute vector stores the owner entity in addition
    			// to the attribute entity itself. 
    			Vector<String> attributeVector = new Vector<String>();
    			attributeVector.add(entity);
    			attributeVector.add(attribute);
    			attributesInCurrentModel.add(attributeVector);
    		}
    		
    		for ( String parent : analyzeModel.getEditor().getParents(entity) ) {
    			// Parent vector stores both the parent and inherited entities' names.
    			Vector<String> parentVector = new Vector<String>();
    			parentVector.add(entity);
    			parentVector.add(parent);
    			parentsInCurrentModel.add(parentVector);
    		}
    	}
    }
    
    /**
     * Method for mapping the submodel for highlight request.
     * @param highlight Submodel to be highlighted.
     * @param drawNewElements Indicates wether or not the new elements are
     *   supposed to be drawn in the highlighted model. New elements are
     *   those that exist in the highlight request but do not exist in the
     *   current analyze model.
     */
    private void mapHighlightRequest(AnalyzeModel highlight, boolean drawNewElements) {
    	for ( String entity : entitiesRequestedForHighlighting ) {
    		for ( String method: highlight.getEditor().getMethodNames(entity) ) {
    			for ( String target: highlight.getEditor().
    					getEntitiesInfluencedByMethod(entity, method) ) {
    				// Method vector stores the method name plus its owner and
    				// target entities.
    				Vector<String> methodVector = new Vector<String>();
    				methodVector.add(entity);
    				methodVector.add(method);
    				methodVector.add(target);
    				methodsRequestedForHighlighting.add(methodVector);
    			}
    		}
    		
    		for ( String attribute : highlight.getEditor().getAttributeNames(entity) ) {
    			// Attribute vector stores the owner entity in addition
    			// to the attribute entity itself. 
    			Vector<String> attributeVector = new Vector<String>();
    			attributeVector.add(entity);
    			attributeVector.add(attribute);
    			attributesRequestedForHighlighting.add(attributeVector);
    		}
    		
    		for ( String parent : highlight.getEditor().getParents(entity) ) {
    			// Parent vector stores both the parent and inherited entities' names.
    			Vector<String> parentVector = new Vector<String>();
    			parentVector.add(entity);
    			parentVector.add(parent);
    			parentsRequestedForHighlighting.add(parentVector);
    		}
    		
    		if ( !entitiesInCurrentModel.contains(entity) && !drawNewElements ) continue;
			entitiesToBeHighlighted.add(entity);
    	}
    }
    
    /**
     * Method for highlighting submodels from the analyze model.
     * @param highlight The submodel to be highlighted. If this argument is
     *   null, then all applied highlights are removed.
     * @param drawNewElements Indicates wether or not those elements
     *   which do not exist in the current analyze model should be
     *   also drawn and highlighted.
     */
    public void highlight(AnalyzeModel highlight, boolean drawNewElements) {
    	// Initialize sets for current entities and their relationships
    	// and then map those entities and their relationships which are
    	// requested to be highlighted.
    	entitiesInCurrentModel = analyzeModel.getEditor().getEntityNames();
    	entitiesRequestedForHighlighting = new HashSet<String>();
    	if ( highlight != null ) entitiesRequestedForHighlighting = highlight.getEditor().getEntityNames();
    	methodsInCurrentModel = new HashSet<Vector<String>>();
    	methodsRequestedForHighlighting = new HashSet<Vector<String>>();
    	parentsInCurrentModel = new HashSet<Vector<String>>();
    	parentsRequestedForHighlighting = new HashSet<Vector<String>>();
    	attributesInCurrentModel = new HashSet<Vector<String>>();
    	attributesRequestedForHighlighting = new HashSet<Vector<String>>();
    	
    	// If highlight is null, then just remove any highlightings that
    	// are currently applied.
    	if ( highlight != null ) {
        	entitiesToBeHighlighted = new HashSet<String>();
        	methodsToBeHighlighted = new HashSet<Vector<String>>();
        	attributesToBeHighlighted = new HashSet<Vector<String>>();
        	parentsToBeHighlighted = new HashSet<Vector<String>>();
        	
        	mapCurrentModel();
        	
        	mapHighlightRequest(highlight, drawNewElements);
        	
        	// Calculate methods to be highlighted.
	    	for ( Vector<String> method : methodsRequestedForHighlighting ) {
	    		if ( !methodsInCurrentModel.contains(method) && !drawNewElements ) continue;
	    		methodsToBeHighlighted.add(method);
	    	}
	    	
	    	// Calculate attributes to be highlighted.
	    	for ( Vector<String> attribute : attributesRequestedForHighlighting ) {
	    		if ( !attributesInCurrentModel.contains(attribute) && !drawNewElements ) continue;
	    		attributesToBeHighlighted.add(attribute);
	    	}
	    	
	    	// Calculate parents to be highlighted.
	    	for ( Vector<String> parent : parentsRequestedForHighlighting ) {
	    		if ( !parentsInCurrentModel.contains(parent) && !drawNewElements ) continue;
	    		parentsToBeHighlighted.add(parent);
	    	}
    	}
    	
    	updateModel(this.analyzeModel);
    	
    	// Destroy class sets associated with highlighting.
    	entitiesInCurrentModel = null;
    	entitiesToBeHighlighted = null;
    	entitiesRequestedForHighlighting = null;
    	methodsInCurrentModel = null;
    	methodsToBeHighlighted = null;
    	methodsRequestedForHighlighting = null;
    	attributesInCurrentModel = null;
    	attributesToBeHighlighted = null;
    	attributesRequestedForHighlighting = null;
    	parentsInCurrentModel = null;
    	parentsToBeHighlighted = null;
    	parentsRequestedForHighlighting = null;
    }
    
    /**
     * Method for highlighting submodels from the analyze model.
     * As default we assume that new entities don't need to be drawn.
     * @param highlight Submodel to be highlighted.
     */
    public void highlight(AnalyzeModel highlight) {
    	highlight(highlight, false);
    }
    
    /**
     * Method for updating the analyze model.
     * @param model New analyze model.
     */
    public void updateModel(AnalyzeModel model) {

        long startTime = System.currentTimeMillis();

        this.analyzeModel = model;

        File dotFile = createDotFile();
        if (dotFile == null) {
            System.err
                    .println("Error in initializing dotFile, exiting updateAnalyzemodel.");
            return;
        }

        File imageFile = runDot(dotFile);
        if (imageFile == null) {
            System.err
                    .println("Error in initializing imageFile, exiting updateAnalyzemodel.");
            return;
        }
        
        // Check if image file exists... just to be sure.
        if (!imageFile.exists()) {
            System.err.println("DotPanel.java: imageFile (" + imageFile.getName() + ") not found.");
            return;
        }

        // If everything went ok, load image from file.
        try {
            this.modelImage = ImageIO.read(imageFile);
        } catch (IOException e) {
            System.err.println("DotPanel.java: Error in reading image from file.");
            e.printStackTrace();
            return;
        }

        this.repaint();

        long timeElapsed = System.currentTimeMillis() - startTime;

        // Remove temp files
        // dotFile.delete();
        // imageFile.delete();

        System.out.println("Panel updated in " + (timeElapsed) + " ms.");
    }
    
    /**
     * This method draws the graph generated by dot to the panel's
     * canvas.
     */
    public void paintComponent(Graphics g) {
    	super.setBackground(colorTheme.getBackgroundColorAsJavaObject());
        super.paintComponent(g);
        int x = 0, y = 0;
        if (this.modelImage != null) {
            x = Math.max(0, (int) Math.floor((this.getWidth() - this.modelImage
                    .getWidth()) / 2));
            y = Math.max(0,
                    (int) Math.floor((this.getHeight() - this.modelImage
                            .getHeight()) / 2));

            g.drawImage(this.modelImage, x, y, this);
            
            this.setPreferredSize(new Dimension(this.modelImage.getWidth(), this.modelImage.getHeight()));
            this.revalidate();
        }
        
    }

}
