package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import ucot.Messages;
import ucot.model.AnalyzeModel;
import ucot.model.ModelEditor;
import ucot.output.Output;

/**
 * Dialog for modifying a single entity's properties in the analyze model.
 * User can modify entity's name, its methods, attributes and parents
 * with this dialog and when modifications are done, the modified analyze
 * model is returned.
 * 
 * Usage:
 *   - first initialize an EntityPropertiesDialog objecs as any other object. 
 *   - then call method:
 *       modifyEntityProperties(Entity to edit, AnalyzeModel)
 *     which returns the modified analyze model.
 * 
 * @author ilanliuk, tujupien.
 */
public class EntityPropertiesDialog extends JDialog {
	
	private String selectedEntityName;
	private JTabbedPane entityPropertiesTabbedPane;
	private AnalyzeModel analyzeModel;
	private boolean verboseMode;
	private JTable methodsTable;
	private JPanel methodsContainerPanel;
	private JScrollPane methodsScrollPane;
	private DefaultTableModel methodsTableModel;
	private JTable attributesTable;
	private JPanel attributesContainerPanel;
	private JScrollPane attributesScrollPane;
	private DefaultTableModel attributesTableModel;
	private JTable parentsTable;
	private JPanel parentsContainerPanel;
	private JScrollPane parentsScrollPane;
	private DefaultTableModel parentsTableModel;
	private JTextField entityName = new JTextField(
			// Default entity name for new entities.
			Messages.getString("EntityPropertiesDialog.0"));
	protected static String DIALOG_HEADER =
		Messages.getString("EntityPropertiesDialog.64");
	protected static String[] METHODS_TABLE_COLUMNS = {
			// Header for method name column.
			Messages.getString("EntityPropertiesDialog.1"),
			// Header for method target column.
			Messages.getString("EntityPropertiesDialog.2")};
	protected static String[] ATTRIBUTES_TABLE_COLUMNS = {
			// Header for attribute name column.
			Messages.getString("EntityPropertiesDialog.3"),
			// Header for to cardinal column.
			Messages.getString("EntityPropertiesDialog.4"),
			// Header for from cardinal column.
			Messages.getString("EntityPropertiesDialog.5")};
	protected static String[] PARENTS_TABLE_COLUMNS = {
			// Header for parents column.
			Messages.getString("EntityPropertiesDialog.6")};
	protected static String DEFAULT_CARDINALITY = "1";
	protected static String CARDINALITY_TOOLTIP =
		// Cardinality column tooltip text.
		Messages.getString("EntityPropertiesDialog.8");
	
	protected static String SELF_POINTER_NAME =
		// Self pointer string.
		Messages.getString("EntityPropertiesDialog.12");
	protected static String LOG_PREFIX =
		// Prefix for log messages.
		Messages.getString("EntityPropertiesDialog.13");
	protected static String LOG_CHANGED_PROPERTIES =
		// Localized string to for "changed properties".
		Messages.getString("EntityPropertiesDialog.14");
	protected static String LOG_ENTITY_NAME =
		// Entity name description in local language.
		Messages.getString("EntityPropertiesDialog.15");
	protected static String LOG_CHANGED_TO =
		// Phrase "changed to" in local language.
		Messages.getString("EntityPropertiesDialog.16");
	// Other log messages localized:
	protected static String LOG_NEW_PARENTS =
		Messages.getString("EntityPropertiesDialog.17");
	protected static String LOG_DELETED_PARENTS =
		Messages.getString("EntityPropertiesDialog.18");
	protected static String LOG_NEW_ATTRIBUTES =
		Messages.getString("EntityPropertiesDialog.19");
	protected static String LOG_DELETED_ATTRIBUTES =
		Messages.getString("EntityPropertiesDialog.20");
	protected static String LOG_ATTRIBUTE_CARDINALITIES_CHANGED =
		Messages.getString("EntityPropertiesDialog.21");
	protected static String LOG_NEW_METHODS =
		Messages.getString("EntityPropertiesDialog.22");
	protected static String LOG_DELETED_METHODS =
		Messages.getString("EntityPropertiesDialog.23");
	protected static String LOG_CHANGED_METHODS =
		Messages.getString("EntityPropertiesDialog.24");
	// Button labels localized.
	protected static String NEW_METHOD_BUTTON = 
		Messages.getString("EntityPropertiesDialog.69");
	protected static String DELETE_METHOD_BUTTON = 
		Messages.getString("EntityPropertiesDialog.71");
	protected static String NEW_ATTRIBUTE_BUTTON = 
		Messages.getString("EntityPropertiesDialog.73");
	protected static String DELETE_ATTRIBUTE_BUTTON = 
		Messages.getString("EntityPropertiesDialog.75");
	protected static String NEW_PARENT_BUTTON = 
		Messages.getString("EntityPropertiesDialog.77");
	protected static String DELETE_PARENT_BUTTON = 
		Messages.getString("EntityPropertiesDialog.79");
	protected static String MAIN_OK_BUTTON = 
		Messages.getString("EntityPropertiesDialog.81");
	protected static String MAIN_CANCEL_BUTTON = 
		Messages.getString("EntityPropertiesDialog.83");
	protected static String DELETE_ENTITY_BUTTON = 
		Messages.getString("EntityPropertiesDialog.85");
	// Tab headers localized.
	protected static String METHODS_TAB_HEADER = 
		Messages.getString("EntityPropertiesDialog.65");
	protected static String PARENTS_TAB_HEADER =
		Messages.getString("EntityPropertiesDialog.66");
	protected static String ATTRIBUTES_TAB_HEADER = 
		Messages.getString("EntityPropertiesDialog.67");
	protected static String MAIN_PROPERTIES_HEADER =
		Messages.getString("EntityPropertiesDialog.68");
	// Button action identifiers.
	protected final static String NEW_METHOD_BUTTON_ACTION = "NEW_METHOD";
	protected final static String DELETE_METHOD_BUTTON_ACTION = "DELETE_METHOD";
	protected final static String NEW_PARENT_BUTTON_ACTION = "NEW_PARENT";
	protected final static String DELETE_PARENT_BUTTON_ACTION = "DELETE_PARENT";
	protected final static String NEW_ATTRIBUTE_BUTTON_ACTION = "NEW_ATTRIBUTE";
	protected final static String DELETE_ATTRIBUTE_BUTTON_ACTION = "DELETE_ATTRIBUTE";
	protected final static String MAIN_OK_BUTTON_ACTION = "MAIN_OK";
	protected final static String MAIN_CANCEL_BUTTON_ACTION = "MAIN_CANCEL";
	protected final static String DELETE_ENTITY_BUTTON_ACTION = "DELETE_ENTITY";
	
	/**
	 * This method creates a new JComboBox Cell Editor for a JTable element
	 * from the given String array. Duplicate entries from given array will
	 * be filtered and the array is sorted to alphabetical order.
	 * @param values Available options in the JComboBox.
	 * @param allowSelf Allow self pointer in the list.
	 * @param addEmpty Create an empty item to the beginning of the list.
	 * @return JComboBox table cell editor as a DefaultCellEditor.
	 */
	protected DefaultCellEditor createComboBoxCellEditor(String[] values, boolean allowSelf,
			boolean addEmpty) {
		JComboBox result = new JComboBox();
		List<String> valuesList = Arrays.asList(values);
		// Sort values to alphabetical order.
		Collections.sort(valuesList);
		// Create self pointer and empty items to the beginning of the list
		// if required.
		if ( addEmpty ) result.addItem("");
		if ( allowSelf ) result.addItem(SELF_POINTER_NAME);
		// Add values to the JComboBox item list and filter out duplicates.
		for ( int i = 0; i < valuesList.size(); i++ ) {
			if (valuesList.get(i).equals(this.selectedEntityName) ) continue;
			if ( i >= 0 && result.getItemCount() > 0 &&
					!valuesList.get(i).equals(result.getItemAt(result.getItemCount()-1)) )
				result.addItem(valuesList.get(i));
			else if ( result.getItemCount() == 0 )
				result.addItem(valuesList.get(i));
		}
		// Spawn a new DefaultCellEditor from the resulting JComboBox.
		return new DefaultCellEditor(result);
	}
	
	/**
	 * This method creates a new JComboBox Cell Editor for a JTable element
	 * from the given String array. Duplicate entries from given array will
	 * be filtered and the array is sorted to alphabetical order. Calling
	 * this method will both allow self pointers and create an empty item
	 * to the beginning of the drop down list.
	 * @param values Available options in the JComboBox.
	 * @return JComboBox table cell editor as a DefaultCellEditor.
	 */
	protected DefaultCellEditor createComboBoxCellEditor(String[] values) {
		return createComboBoxCellEditor(values, true, true);
	}
	
	/**
	 * This method creates a new JComboBox Cell Editor for a JTable element
	 * from the given String array. Duplicate entries from given array will
	 * be filtered and the array is sorted to alphabetical order. Calling
	 * this method will also create an empty item to the beginning of the
	 * drop down list.
	 * @param values Available options in the JComboBox.
	 * @param allowSelf Allow self pointer in the list.
	 * @return JComboBox table cell editor as a DefaultCellEditor.
	 */
	protected DefaultCellEditor createComboBoxCellEditor(String[] values, boolean allowSelf) {
		return createComboBoxCellEditor(values, allowSelf, true);
	}
	
	/**
	 * This inline class provides a syntax verifier for the cardinalities.
	 * Cardinalities must be in the following format (regular expression):
	 *   [0-9n]+\.{2,3}[0-9nm]+
	 *     - or -
	 *   [0-9n]+
	 */
	protected static class CardinalityVerifier extends InputVerifier {
		public boolean verify(JComponent input) {
			if ( input instanceof JTextField ) {
				// Get text contents (cardinality string) from the
				// input component.
				JTextField inputTextField = (JTextField)input;
				String inputString = inputTextField.getText();
				// Initialize regular expression patterns and matchers.
				Pattern pattern1 = Pattern.compile("([0-9]+|n)\\.{2,3}([0-9]+|[nm])");
				Pattern pattern2 = Pattern.compile("([0-9]+|[nm])");
				Matcher matcher1 = pattern1.matcher(inputString);
				Matcher matcher2 = pattern2.matcher(inputString);
				return matcher1.matches() || matcher2.matches();
			}
			return false;
		}
	}
	
	/**
	 * This class implements a focus listener to the cardinality editor.
	 * It forces the cardinality to be valid.
	 */
	private class CardinalityEditorFocusListener implements FocusListener {
		private JTextField editor;
		private String oldCardinality;
		private JTable associatedTable;
		private TableModel associatedTableModel;
		private int tableRow;
		private int tableCol;
		
		/**
		 * Constructor for CardinalityEditorFocusListener.
		 * @param owner Owner JTextField object of this listener.
		 * @param associatedTable JTable the owner is associated with.
		 *   replaces any non-standard cardinality entered.
		 */
		public CardinalityEditorFocusListener(JTextField owner, JTable associatedTable) {
			this.editor = owner;
			this.associatedTable = associatedTable;
			this.associatedTableModel = associatedTable.getModel();
		}
		
		/**
		 * This method forces the entered cardinality to match the
		 * default cardinality if the entered cardinality is invalid.
		 */
		/* (non-Javadoc)
		 * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
		 */
		public void focusLost(FocusEvent e) {
			if ( this.editor != null ) {
				if ( !this.editor.getInputVerifier().verify(this.editor) ) {
				    this.editor.setText(this.oldCardinality);
				}
			}
			associatedTableModel.setValueAt(this.editor.getText(), this.tableRow,
					this.tableCol);
		}
		
		/**
		 * Track the position in the table and save the old value.
		 */
		/* (non-Javadoc)
		 * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
		 */
		public void focusGained(FocusEvent e) {
			this.tableCol = associatedTable.getEditingColumn();
			this.tableRow = associatedTable.getEditingRow();
			if ( associatedTableModel.getValueAt(this.tableRow, this.tableCol)
					instanceof String ) {
				this.oldCardinality = (String)associatedTableModel.getValueAt(
						this.tableRow, this.tableCol);
			} else {
				this.oldCardinality = DEFAULT_CARDINALITY;
			}
		}
	}
	
	/**
	 * Method for creating a cardinality editor as a DefaultCellEditor.
	 * Mainly used for JTables where cardinalities must be edited.
	 * @param associatedTable Table to associate this cardinality editor with.
	 * @return Cardinality editor inherited from a JTextField as a
	 *   DefaultCellEditor object.
	 */
	protected DefaultCellEditor createCardinalityEditor(JTable associatedTable) {
		// Create a JTextField and apply a CardinalityVerifier and
		// cardinality tooltip to it.
		JTextField cardinalityEditor = new JTextField();
		cardinalityEditor.setInputVerifier(new CardinalityVerifier());
		cardinalityEditor.setToolTipText(CARDINALITY_TOOLTIP);
		cardinalityEditor.addFocusListener(new CardinalityEditorFocusListener(
				cardinalityEditor, associatedTable));
		
		return new DefaultCellEditor(cardinalityEditor);
	}
	
	/**
	 * ComponentListener is required to keep the JTable elements correctly
	 * sized in height.
	 */
	ComponentListener resizeListener = new ComponentListener() {
		
		/**
		 * This method resizes all JTable elements in the dialog to
		 * fit the dialog's current size.
		 */
		private void resizeTables() {
			Dimension dim = null;
			// Resize methods table.
			dim = methodsContainerPanel.getSize();
			methodsScrollPane.setPreferredSize(dim);
			methodsTable.invalidate();
			// Resize attributes table.
			dim = attributesContainerPanel.getSize();
			attributesScrollPane.setPreferredSize(dim);
			attributesTable.invalidate();
			// Resize parents table.
			dim = parentsContainerPanel.getSize();
			parentsScrollPane.setPreferredSize(dim);
			parentsTable.invalidate();
		}
		
		/**
		 * When the dialog is resized, all tables must be resized
		 * to fit their parent elements. 
		 */
		public void componentResized(ComponentEvent e) { resizeTables(); }
		public void componentHidden(ComponentEvent e) {}
		public void componentShown(ComponentEvent e) {}
		public void componentMoved(ComponentEvent e) {}
	};
	
	/**
	 * ActionListener for all button events within the dialog.
	 */
	private ActionListener buttonListener = new ActionListener() {
		private ModelEditor editor;
		
		/**
		 * This method updates the entity name if it has changed.
		 */
		private void updateEntityName() {
			if ( !entityName.getText().equals(selectedEntityName) ) {
				
				
				// Uppcase the first letter.
				String newName = entityName.getText().trim();
				char firstLetter = Character.toUpperCase(newName.charAt(0));
				newName = firstLetter + newName.substring(1);
				
				// Entity name was changed.
				if ( verboseMode ) {
					System.out.println(LOG_PREFIX + " - " + LOG_ENTITY_NAME + " '" +
							selectedEntityName + "' " + LOG_CHANGED_TO +
							" '" + newName + "'");
				}
				// Now either add a new entity to the analyze model (new entity) or
				// change the name of an already existing entity (modify entity).
                if ( !analyzeModel.getEditor().containsEntity(selectedEntityName) ) {
                    editor.addEntity(newName);
                } else {
                    editor.changeEntityName(selectedEntityName, newName);
                }
				
				// The corresponding class variable must be updated if
                // entity name was changed.
				selectedEntityName = newName;
			}
		}
		
		/**
		 * This method updates entity parent relationships. 
		 */
		private void updateEntityParents() {
			Set<String> oldParents = editor.getParents(selectedEntityName);
			Set<String> currentParents = new HashSet<String>();
			
			// Initialize current parents.
			for ( int i = 0; i < parentsTableModel.getRowCount(); i++ ) {
				String parentName = ((String) parentsTableModel.getValueAt(i,0)).trim();
				// Paretn cannot be zero length or the object itself cant be its parent.
				if ( parentName.length() > 0 && !parentName.equals(selectedEntityName) ) {
					currentParents.add(parentName);
				}
			}
			
			// Calculate new parents and deleted parents.
			Set<String> newParents = new HashSet<String>();
			newParents.addAll(currentParents);
			newParents.removeAll(oldParents);
			Set<String> deletedParents = new HashSet<String>();
			deletedParents.addAll(oldParents);
			deletedParents.removeAll(currentParents);
			
			if ( verboseMode ) {
				System.out.println(LOG_PREFIX + " - " + LOG_NEW_PARENTS +
						": " + newParents);
				System.out.println(LOG_PREFIX + " - " + LOG_DELETED_PARENTS +
						": " + deletedParents);
			}
			
			// Add new parents to the analyze model.
			for ( String parentName: newParents ) {
				editor.addParent(selectedEntityName, parentName);
			}
			
			// Remove non existing parent relationships from the analyze model.
			for ( String parentName: deletedParents ) {
				editor.removeParent(selectedEntityName, parentName);
			}
		}
		
		/**
		 * This method updates entity attributes.
		 */
		private void updateEntityAttributes() {
			Set<String> oldAttributes = editor.getAttributeNames(selectedEntityName);
			Set<String> currentAttributes = new HashSet<String>();
			// Cardinalities are expressed here as a 3-element String array, which
			// contains {attributeName, fromCardinal, toCardinal}.
			Set<String[]> oldCardinalities = new HashSet<String[]>();
			Set<String[]> currentCardinalities = new HashSet<String[]>();
			
			// Initialize old attributes and cardinalities sets.
			for ( String attribute: oldAttributes ) {
				String[] cardinality = {attribute,
						editor.getAttributeFromCardinal(selectedEntityName, attribute),
						editor.getAttributeToCardinal(selectedEntityName, attribute)};
				oldCardinalities.add(cardinality);
			}
			
			// Initialize current attributes and cardinalities sets.
			for ( int i = 0; i < attributesTableModel.getRowCount(); i++ ) {
				String attributeName = ((String) attributesTableModel.getValueAt(i,0)).trim();
				if ( attributeName.equals(SELF_POINTER_NAME) ){
					attributeName = selectedEntityName;
				}
				String attributeToCardinality = ((String)
						attributesTableModel.getValueAt(i,1)).trim();
				String attributeFromCardinality = ((String)
						attributesTableModel.getValueAt(i,2)).trim();
				
				if ( attributeName.length() > 0 ) {
					currentAttributes.add(attributeName);
					String[] cardinality = {attributeName, attributeFromCardinality,
							attributeToCardinality};
					currentCardinalities.add(cardinality);
				}
			}
			
			// Calculate new attributes and deleted attributes.
			Set<String> newAttributes = new HashSet<String>();
			newAttributes.addAll(currentAttributes);
			newAttributes.removeAll(oldAttributes);
			Set<String> deletedAttributes = new HashSet<String>();
			deletedAttributes.addAll(oldAttributes);
			deletedAttributes.removeAll(currentAttributes);
			
			if ( verboseMode ) {
				System.out.println(LOG_PREFIX + " - " + LOG_NEW_ATTRIBUTES +
						": " + newAttributes);
				System.out.println(LOG_PREFIX + " - " + LOG_DELETED_ATTRIBUTES +
						": " + deletedAttributes);
			}
			
			// Add new attributes to the analyze model.
			for ( String attributeName : newAttributes ) {
				editor.addAttribute(selectedEntityName, attributeName);
			}
			
			// Remove non-existing attribute relationships from the analyze model.
			for ( String attributeName : deletedAttributes ) {
				editor.removeAttribute(selectedEntityName, attributeName);
			}
			
			// Set cardinalities.
			Set<String> changedAttributes = new HashSet<String>();
			for ( String attribute : currentAttributes ) {
				String fromCardinality = "1", toCardinality = "1";
				for ( String[] oldCardinality : oldCardinalities ) {
					if ( oldCardinality[0].equals(attribute) ) {
						fromCardinality = oldCardinality[1];
						toCardinality = oldCardinality[2];
						break;
					}
				}
				for ( String[] newCardinality : currentCardinalities ) {
					if ( newCardinality[0].equals(attribute) ) {
						if ( !fromCardinality.equals(newCardinality[1]) ||
								!toCardinality.equals(newCardinality[2]) ) {
							changedAttributes.add(attribute);
						}
						fromCardinality = newCardinality[1];
						toCardinality = newCardinality[2];
						break;
					}
				}
				editor.setAttributeFromCardinal(selectedEntityName, attribute, fromCardinality);
				editor.setAttributeToCardinal(selectedEntityName, attribute, toCardinality);
			}
			
			if ( verboseMode ) {
				System.out.println(LOG_PREFIX + " - " + LOG_ATTRIBUTE_CARDINALITIES_CHANGED +
						changedAttributes);
			}
		}
		
		/**
		 * This method updates entity methods.
		 */
		private void updateEntityMethods() {
			// Initialize old methods and current methods sets.
			Map<String, Set<String>> methodEntityRelations = new HashMap<String, Set<String>>();
			Set<String> oldMethods = analyzeModel.getEditor().getMethodNames(selectedEntityName);
			Set<String> currentMethods = new HashSet<String>();
			for ( int i = 0; i < methodsTableModel.getRowCount(); i++ ) {
				String methodName = ((String) methodsTableModel.getValueAt(i,0)).trim();
				String entityName = ((String) methodsTableModel.getValueAt(i,1)).trim();
				if ( entityName.equals(SELF_POINTER_NAME) ) entityName = selectedEntityName;
				
				// Check wether row has empty cells or not.                    
				if ( methodName.length() > 0 ) {
					currentMethods.add(methodName);
					
					Set<String> list = methodEntityRelations.get(methodName);
					if ( null == list ){ 
						list = new HashSet<String>();
						methodEntityRelations.put(methodName,list);
					}
					if ( entityName.length() > 0 ){
						list.add(entityName);
					}
				}
			}
			
			// Calculate new methods, deleted methods and changed methods.
			Set<String> newMethods = new HashSet<String>();
			newMethods.addAll(currentMethods);
			newMethods.removeAll(oldMethods);
			Set<String> deletedMethods = new HashSet<String>();
			deletedMethods.addAll(oldMethods);
			deletedMethods.removeAll(currentMethods);
			Set<String> possiblyChangedMethods = new HashSet<String>();
			possiblyChangedMethods.addAll(oldMethods);
			possiblyChangedMethods.removeAll(deletedMethods);
			
			// Add new methods to the analyze model.
			for ( String methodName : newMethods ) {
				editor.addMethod(selectedEntityName, methodName);
                Set<String> currentEntities = methodEntityRelations.get(methodName);
                for ( String entity : currentEntities ) {
                    editor.addEntityInfluenceByMethod(selectedEntityName,methodName, entity);
                }
            }
			
			// Remove non-existing method relationships from the analyze model.
			for ( String methodName : deletedMethods ) {
				editor.removeMethod(selectedEntityName, methodName);
			}
			
			// Update changed methods from the analyze model.
			for ( String methodName : possiblyChangedMethods ) {
				Set<String> oldEntities = editor.getEntitiesInfluencedByMethod(selectedEntityName,
						methodName);
				Set<String> currentEntities = methodEntityRelations.get(methodName);
				Set<String> missingEntities = new HashSet<String>();
				missingEntities.addAll(currentEntities);
				missingEntities.removeAll(oldEntities);
				
				Set<String> unnecessaryEntities = new HashSet<String>();
				unnecessaryEntities.addAll(oldEntities);
				unnecessaryEntities.removeAll(currentEntities);
				
				for(String entity : missingEntities){
					editor.addEntityInfluenceByMethod(selectedEntityName, methodName, 
							entity);
				}
				
				for(String entity : unnecessaryEntities){
					editor.removeEntityInfluenceByMethod(selectedEntityName, methodName, 
							entity);
				}
			}
			
			if ( verboseMode ) {
				System.out.println(LOG_PREFIX + " - " + LOG_NEW_METHODS  + ": " + 
						newMethods);
				System.out.println(LOG_PREFIX + " - " + LOG_DELETED_METHODS + ": " +
						deletedMethods);
				System.out.println(LOG_PREFIX + " - " + LOG_CHANGED_METHODS + ": " +
						possiblyChangedMethods);
			}
		}
		
		/**
		 * Action performed: User clicked the main OK button.
		 */
		private void mainOKButtonClicked() {
			// Stop cell editing in all tables before doing anything else.
			if ( methodsTable.isEditing() ) 
				methodsTable.getCellEditor().stopCellEditing();
			if ( attributesTable.isEditing() )
				attributesTable.getCellEditor().stopCellEditing();
			if ( parentsTable.isEditing() )
				parentsTable.getCellEditor().stopCellEditing();
			
			// Check out what has changes have been made
			// and make same changes to analyze model.
			
			if ( verboseMode ) {
				System.out.println("--------------------------------------------------");
				System.out.println(LOG_PREFIX + LOG_CHANGED_PROPERTIES);
			}
			
			analyzeModel.updationStartedSignal();
			
			// Update entity name.
			updateEntityName();
			
			// Update parent classes for current entity.
			updateEntityParents();
			
			// Update entity's attributes and their cardinalities. 
			updateEntityAttributes();
			
			// Update changed methods to the analyze model.
			updateEntityMethods();
			
			analyzeModel.readySignal();
			if ( verboseMode ) {
				System.out.println("--------------------------------------------------");
			}
			
			// Vanish dialog.
			setVisible(false);
		}
		
		/**
		 * Action performed: User clicked the main cancel button.
		 */
		private void mainCancelButtonClicked() {
			// This seems to be enough to close modal dialog,
			// don't know for sure (ilari)
            
            // Checked here if new entity has already 
            // been created that it is removed..
			
			// Vanish dialog.
			setVisible(false);
		}
		
		/**
		 * Action performed: User clicked the 'delete entity' button.
		 */
		private void deleteEntityButtonClicked() {
			// Delete the selected entity and all relations 
			// from and to it.
			
			// model.removeItem(selectedEntity);
			
			// Vanish dialog.
			setVisible(false);
		}
		
		/**
		 * Action performed: User clicked the 'new method' button.
		 *
		 */
		private void newMethodButtonClicked() {
			// Add new row to method table.
			Vector<String> row = new Vector<String>();
			row.add("");
			row.add("");
			methodsTableModel.addRow(row);
		}
		
		/**
		 * Action performed: User clicked the 'delete method' button.
		 */
		private void deleteMethodButtonClicked() {
			// Stop cell editing in the table.
			if ( methodsTable.isEditing() )
				methodsTable.getCellEditor().cancelCellEditing();
			// Delete selected row from methods table.
			if ( methodsTable.getSelectedRow() < 0 ) return;
			methodsTableModel.removeRow(methodsTable.getSelectedRow());
		}
		
		/**
		 * Action performed: User clicked the 'new attribute' button.
		 */
		private void newAttributeButtonClicked() {
			// Add new attribute relation to attributes table.
			Vector<String> row = new Vector<String>();
			row.add("");
			row.add("1");
			row.add("1");
			attributesTableModel.addRow(row);
		}
		
		/**
		 * Action performed: User clicked the 'delete attribute' button.
		 */
		private void deleteAttributeButtonClicked() {
			// Stop cell editing.
			if ( attributesTable.isEditing() )
				attributesTable.getCellEditor().stopCellEditing();
			// Delete selected row from relations table.
			if ( attributesTable.getSelectedRow() < 0 ) return; 
			attributesTableModel.removeRow(attributesTable.getSelectedRow());
		}
		
		/**
		 * Action performed: User clicked the 'new parent' button.
		 */
		private void newParentButtonClicked() {
			// Add new parent to the parents table.
			Vector<String> row = new Vector<String>();
			row.add("");
			parentsTableModel.addRow(row);
		}
		
		/**
		 * Action performed: User clicked the 'delete parent' button.
		 */
		private void deleteParentButtonClicked() {
			// Stop cell editing.
			if ( parentsTable.isEditing() )
				parentsTable.getCellEditor().stopCellEditing();
			// Delete selected row from the parents table.
			if ( parentsTable.getSelectedRow() < 0 ) return;
			parentsTableModel.removeRow(parentsTable.getSelectedRow());
		}
		
		/**
		 * This method is called when an action is performed and it
		 * calls the corresponding methods to handle the action. 
		 */
		public void actionPerformed(ActionEvent e) {
			editor = analyzeModel.getEditor();
			
			if ( MAIN_OK_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				mainOKButtonClicked(); return;
			}
			
			if ( MAIN_CANCEL_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				mainCancelButtonClicked(); return;
			}
			
			if ( DELETE_ENTITY_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				deleteEntityButtonClicked(); return;
			}
			
			if ( NEW_METHOD_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				newMethodButtonClicked(); return;
			}
			
			if ( DELETE_METHOD_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				deleteMethodButtonClicked(); return;
			}
			
			if ( NEW_ATTRIBUTE_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				newAttributeButtonClicked(); return;
			}
			
			if ( DELETE_ATTRIBUTE_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				deleteAttributeButtonClicked(); return;
			}
			
			if ( NEW_PARENT_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				newParentButtonClicked(); return;
			}
			
			if ( DELETE_PARENT_BUTTON_ACTION.equals(e.getActionCommand()) ) {
				deleteParentButtonClicked(); return;
			}
		}
		
	};
	
	/**
	 * EntityPropertiesDialog constructor. This creates the whole dialog,
	 * but the actual contents of the all fields will be set later in
	 * the initialization method.
	 * @param owner This dialog's owner component.
	 * @param beVerbose Boolean variable which indicates wether this dialog
	 *   should be verbose or not.
	 * @throws HeadlessException Exception is thrown if the superclass
	 *   initialization goes wrong.
	 */
	public EntityPropertiesDialog(JFrame owner, boolean beVerbose) throws HeadlessException {
		super(owner, DIALOG_HEADER, true);
		this.verboseMode = beVerbose;
		
		this.setPreferredSize(new Dimension(500, 500));
		this.setLayout(new BorderLayout());
		
		// Create all required panels and set layouts for them.
		entityPropertiesTabbedPane = new JTabbedPane();
		JPanel mainButtonsPanel = new JPanel();
		JPanel mainPropertiesPanel = new JPanel();
		mainPropertiesPanel.setLayout(new BorderLayout());
		JPanel methodsTabPanel = new JPanel();
		JPanel methodsButtonsPanel = new JPanel();
		methodsTabPanel.setLayout(new BorderLayout());
		methodsContainerPanel = new JPanel();
		methodsContainerPanel.setLayout(new BorderLayout());
		JPanel parentsTabPanel = new JPanel();
		JPanel parentsButtonsPanel = new JPanel();
		parentsTabPanel.setLayout(new BorderLayout());
		parentsContainerPanel = new JPanel();
		parentsContainerPanel.setLayout(new BorderLayout());
		JPanel attributesButtonsPanel = new JPanel();
		JPanel attributesTabPanel = new JPanel();
		attributesTabPanel.setLayout(new BorderLayout());
		attributesContainerPanel = new JPanel();
		attributesContainerPanel.setLayout(new BorderLayout());
				
		// Initialize tables and table models.
		methodsTableModel = new DefaultTableModel();
		methodsTable = new JTable(methodsTableModel);
		attributesTableModel = new DefaultTableModel();
		attributesTable = new JTable(attributesTableModel);
		parentsTableModel = new DefaultTableModel();
		parentsTable = new JTable(parentsTableModel);
		
		methodsTable.setRowSelectionAllowed(false);
		attributesTable.setRowSelectionAllowed(false);
		parentsTable.setRowSelectionAllowed(false);
		
		// Initialize scroll panes for tables.
		attributesScrollPane = new JScrollPane(attributesTable);
		methodsScrollPane = new JScrollPane(methodsTable);
		parentsScrollPane = new JScrollPane(parentsTable);
		
		// Divide the dialog into four sections. Top section for entity properties,
		// left (center) section for entity's methods table and bottom section for
		// general dialog buttons.
		this.getContentPane().add(mainButtonsPanel, BorderLayout.SOUTH);
		this.getContentPane().add(mainPropertiesPanel, BorderLayout.NORTH);
		this.getContentPane().add(entityPropertiesTabbedPane, BorderLayout.CENTER);
		entityPropertiesTabbedPane.addTab(METHODS_TAB_HEADER, methodsTabPanel);
		entityPropertiesTabbedPane.addTab(PARENTS_TAB_HEADER, parentsTabPanel);
		entityPropertiesTabbedPane.addTab(ATTRIBUTES_TAB_HEADER, attributesTabPanel);
		
		// Initialize relations table and its containers.
		attributesTabPanel.add(attributesContainerPanel, BorderLayout.CENTER);
		attributesTabPanel.add(attributesButtonsPanel, BorderLayout.SOUTH);
		attributesContainerPanel.add(attributesScrollPane, BorderLayout.NORTH);
		// Initialize methods table and its containers.
		methodsTabPanel.setPreferredSize(new Dimension(350,250));
		methodsTabPanel.add(methodsContainerPanel, BorderLayout.CENTER);
		methodsTabPanel.add(methodsButtonsPanel, BorderLayout.SOUTH);
		methodsContainerPanel.add(methodsScrollPane, BorderLayout.NORTH);
		// Initialize parents table and its containers.
		parentsTabPanel.add(parentsContainerPanel, BorderLayout.CENTER);
		parentsTabPanel.add(parentsButtonsPanel, BorderLayout.SOUTH);
		parentsContainerPanel.add(parentsScrollPane, BorderLayout.NORTH);
		
		// Initialize entity properties' container.
		mainPropertiesPanel.setBorder(BorderFactory.createTitledBorder(MAIN_PROPERTIES_HEADER));
		mainPropertiesPanel.add(entityName);
		
		// Create required buttons. -->
		JButton b = new JButton(NEW_METHOD_BUTTON);
		b.setActionCommand(NEW_METHOD_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		methodsButtonsPanel.add(b);
		b = new JButton(DELETE_METHOD_BUTTON);
		b.setActionCommand(DELETE_METHOD_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		methodsButtonsPanel.add(b);
		
		b = new JButton(NEW_ATTRIBUTE_BUTTON);
		b.setActionCommand(NEW_ATTRIBUTE_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		attributesButtonsPanel.add(b);
		b = new JButton(DELETE_ATTRIBUTE_BUTTON);
		b.setActionCommand(DELETE_ATTRIBUTE_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		attributesButtonsPanel.add(b);
		
		b = new JButton(NEW_PARENT_BUTTON);
		b.setActionCommand(NEW_PARENT_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		parentsButtonsPanel.add(b);
		b = new JButton(DELETE_PARENT_BUTTON);
		b.setActionCommand(DELETE_PARENT_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		parentsButtonsPanel.add(b);
		
		b = new JButton(MAIN_OK_BUTTON);
		b.setActionCommand(MAIN_OK_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		mainButtonsPanel.add(b);
		b = new JButton(MAIN_CANCEL_BUTTON);
		b.setActionCommand(MAIN_CANCEL_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		mainButtonsPanel.add(b);
		b = new JButton(DELETE_ENTITY_BUTTON);
		b.setActionCommand(DELETE_ENTITY_BUTTON_ACTION);
		b.addActionListener(buttonListener);
		mainButtonsPanel.add(b);
		// <-- Button creation section ends here.
		
		this.pack();
	}
	
	/**
	 * Default constructor for EntityPropertiesDialog. This constructor
	 * assumes that the dialog is not supposed to be in verbose mode.
	 * @param owner Owner component of this dialog.
	 * @throws HeadlessException Exception is thrown if something goes
	 *   wrong with the superclass initialization.
	 */
	public EntityPropertiesDialog(JFrame owner) throws HeadlessException {
		this(owner, false);
	}
	
	/**
	 * Method to spawn the EntityPropertiesDialog and stay modal
	 * until user closes it. Modified analyze model is returned.
	 * @param entity Entity to edit.
	 * @param model Analyze model to edit.
	 * @return Modified analyze model as an AnalyzeModel object.
	 */
	public AnalyzeModel modifyEntityProperties(String entityName, AnalyzeModel model) {
		
		this.selectedEntityName = entityName;
		this.analyzeModel = model;
		
		initializeDialog();
		
		Output output = new Output();
		// This shows the modal dialog, and stops here until
		// the dialog is closed.        
		output.output(this.analyzeModel, null);
		this.setVisible(true);
		output.output(this.analyzeModel, null);
		
		// The analyzemodel is modified when OK button is clicked
		// in the buttonListener.
		
		// Return the modified model.
		return this.analyzeModel;
	}
	
	/**
	 * This method initializes the dialog and all its dynamic components.
	 */
	private void initializeDialog() {
		// Initialize table models by creating the required number of columns.
		if ( methodsTableModel.getColumnCount() != METHODS_TABLE_COLUMNS.length )
			for ( int i = 0; i < METHODS_TABLE_COLUMNS.length; i++ )
				methodsTableModel.addColumn(METHODS_TABLE_COLUMNS[i]);
		if ( attributesTableModel.getColumnCount() != ATTRIBUTES_TABLE_COLUMNS.length )
			for ( int i = 0; i < ATTRIBUTES_TABLE_COLUMNS.length; i++ )
				attributesTableModel.addColumn(ATTRIBUTES_TABLE_COLUMNS[i]);
		if ( parentsTableModel.getColumnCount() != PARENTS_TABLE_COLUMNS.length )
			for ( int i = 0; i < PARENTS_TABLE_COLUMNS.length; i++ )
				parentsTableModel.addColumn(PARENTS_TABLE_COLUMNS[i]);
		
		// Store special table columns in local variables. 
		TableColumn attributesColumn = attributesTable.getColumnModel().getColumn(0);
		TableColumn attributesToCardinalityColumn = attributesTable.getColumnModel().getColumn(1);
		TableColumn attributesFromCardinalityColumn = attributesTable.getColumnModel().getColumn(2);
		TableColumn methodsColumn = methodsTable.getColumnModel().getColumn(1);
		TableColumn parentsColumn = parentsTable.getColumnModel().getColumn(0);
		
		// Set focus to the first tab.
		entityPropertiesTabbedPane.setSelectedIndex(0);

		// Add available entities to entitiesArray which is used for generating
		// the ComboBox editor for some columns. Also set the cardinality editors
		// to cardinality columns.
		Set<String> entities = analyzeModel.getEditor().getEntityNames();
		String[] entitiesArray = new String[entities.size()];
		entitiesArray = entities.toArray(entitiesArray);
		attributesColumn.setCellEditor(createComboBoxCellEditor(entitiesArray, true));
		attributesToCardinalityColumn.setCellEditor(createCardinalityEditor(
				attributesTable));
		attributesFromCardinalityColumn.setCellEditor(createCardinalityEditor(
				attributesTable));
		methodsColumn.setCellEditor(createComboBoxCellEditor(entitiesArray, true));
		parentsColumn.setCellEditor(createComboBoxCellEditor(entitiesArray, false));
		
		entityName.setText(selectedEntityName);
		
		// Erase all data from old table models.
		for ( int i = methodsTableModel.getRowCount()-1; i >= 0; i-- )
			methodsTableModel.removeRow(i);
		for ( int i = attributesTableModel.getRowCount()-1; i >= 0; i-- )
			attributesTableModel.removeRow(i);
		for ( int i = parentsTableModel.getRowCount()-1; i >= 0; i-- )
			parentsTableModel.removeRow(i);
		
		// Load table data from analyze model.
		
		Set<String> methods = analyzeModel.getEditor().getMethodNames(selectedEntityName);
		
		// Add methods to the corresponding table.
		for (String method : methods ) {
			Set<String> targets = analyzeModel.getEditor().
			getEntitiesInfluencedByMethod(selectedEntityName, method);
			if (targets.size() > 0){
				for (String target: targets ) {
					Vector<String> row = new Vector<String>();
					row.addElement(method);
					row.addElement(target);
					methodsTableModel.addRow(row);
				}
			} else {
				Vector<String> row = new Vector<String>();
				row.addElement(method);
				row.addElement("");
				methodsTableModel.addRow(row);        		
			}
		}
		
		Set<String> attributes = analyzeModel.getEditor().getAttributeNames(selectedEntityName);
		
		// Add attributes to the corresponding attributes table.
		for (String attribute : attributes ) {
			Vector<String> row = new Vector<String>();
			row.addElement(attribute);
			row.addElement(analyzeModel.getEditor().
					getAttributeToCardinal(selectedEntityName, attribute));
			row.addElement(analyzeModel.getEditor().
					getAttributeFromCardinal(selectedEntityName, attribute));
			attributesTableModel.addRow(row);
		}
		
		Set<String> parents = analyzeModel.getEditor().getParents(selectedEntityName);
		
		// Add parents to the corresponding table.
		for (String parent: parents ) {
			Vector<String> row = new Vector<String>();
			row.addElement(parent);
			parentsTableModel.addRow(row);
		}
		
		// JTables act in a quite extraordinary way what comes to resizing.
		// That's why a resize listener is required to keep an eye on the
		// dialog dimensions and resize thos JTable elements when required.
		this.addComponentListener(resizeListener);
		resizeListener.componentResized(null);
	}
	
	/**
	 * Constructs, initializes and spawns an entity properties dialog for
	 * the given entity in the given analyze model. The given model is
	 * updated with the user's modifications and then returned.
	 * @param owner Owner component of the to generated entity properties dialog.
	 * @param entityName The name of the entity to be edited.
	 * @param model The analyze model which is being modified.
	 * @return Modified analyze model as an AnalyzeModel object.
	 */
	public static AnalyzeModel showDialog(JFrame owner, String entityName, AnalyzeModel model) {
		EntityPropertiesDialog entityPropertiesDialog = new EntityPropertiesDialog(owner);
		return entityPropertiesDialog.modifyEntityProperties(entityName, model);
	}
}
