package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.net.URL;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;

import ucot.core.ControlInterface;
import ucot.input.UseCaseCollection;
import ucot.output.OutputInterface;
import ucot.ui.UIInterface;
import ucot.ui.UseCasePanelInterface;
import ucot.ui.gui.DotPanel.ExportImageType;
import ucot.ui.gui.tree.AnalyzeModelTree;
import ucot.ui.gui.tree.usecasetree.UseCaseTree;

/**
 * Simple graphical user interface for UCOT core which uses dot (in DotPanel) to
 * draw graphics. This class is inherited from JFrame and implements
 * the UIInterface defined in UCOT package.
 * 
 * @author ilanliuk, tujupien.
 */
/**
 * @author tujupien
 *
 */
public class GraphicalUI extends JFrame implements UIInterface {
    private ControlInterface core = null;
    private ActionListener menuListener;
    private AnalyzeModelTree analyzeModelTree;
    private JScrollPane modelTreeScrollPane;
    private UseCaseTree useCaseTree;
    private JScrollPane useCaseTreeScrollPane;
    private DotPanel dotPanel;
    private JScrollPane dotScrollPane;
    private UseCasePanelInterface useCasePanel = 
    	new SimpleUseCasePanel();
    private JScrollPane useCasePanelScrollPane = 
    	new JScrollPane((JPanel)useCasePanel);
    private EntityPropertiesDialog entityPropertiesDialog = 
    	new EntityPropertiesDialog(this);
    
    protected static String PROGRAM_TITLE = 
    	"UCOT";
    protected static String DEFAULT_ERROR_HEADER = 
    	"Error";
    protected static String DEFAULT_QUESTION_HEADER = 
    	"Question";
    protected static String CLEAR_MODEL_QUESTION_HEADER = 
    	"Clear model";
    protected static String CLEAR_MODEL_QUESTION =
    	"Are you sure you want to clear\nthe entire analyze model?";
    protected static String QUIT_QUESTION_HEADER =
    	"Quit";
    protected static String QUIT_QUESTION =
    	"Are you sure you want to quit\nand lose all unsaved changes?";
    protected static String LOG_PREFIX = 
    	"GUI: ";
    protected static String LOG_FILE_LOAD_TRIGGERED = 
    	"User wants to load a file.";
    protected static String LOG_QUIT_PROGRAM_TRIGGERED = 
    	"User wants to quit program.";
    protected static String LOG_QUIT_PROGRAM = 
    	"Shutting down...";
    protected static String LOG_CLEAR_MODEL_TRIGGERED = 
    	"User wants to clear the entire analyze model.";
    protected static String LOG_SETTINGS_TRIGGERED =
    	"User wants to change program settings.";
    protected static String LOG_EXPORT_AS_IMAGE_TRIGGERED =
    	"Exporting analyze model as image.";
    protected static String LOG_EXPORT_DONE_MESSAGE =
    	"Analyze model successfully exported.";
    protected static String LOG_ANALYZE_MODEL_LOADED =
    	"Analyze model successfully loaded.";
    protected static String LOG_USE_CASES_LOADED =
    	"Use cases have been successfully loaded.";
    protected static String LOG_USE_CASE_LOADING_EXCEPTION =
    	"No available use cases or the file format is not supported.";
    protected static String LOG_USE_CASES_ADDED =
    	"Core told us that usecase was added to analyzemodel";
    protected static String FILE_MENU_CAPTION =
    	"File";
    protected static String OPEN_MENU_CAPTION = 
    	"Open";
    protected static String EXPORT_MENU_CAPTION = 
    	"Export...";
    protected static String EXPORT_AS_IMAGE_MENU_CAPTION = 
    	"Export as Image...";
    protected static String CLEAR_MODEL_MENU_CAPTION =
    	"Clear Model...";
    protected static String QUIT_MENU_CAPTION = 
    	"Quit";
    protected static String PROGRAM_MENU_CAPTION = 
    	"Program";
    protected static String SETTINGS_MENU_CAPTION =
    	"Settings...";
    
    protected static String OPEN_FILE_MENU_ACTION = "OPEN_FILE";
    protected static String EXPORT_MENU_ACTION = "EXPORT";
    protected static String QUIT_MENU_ACTION = "QUIT_PROGRAM";
    protected static String CLEAR_MODEL_MENU_ACTION = "CLEAR_MODEL";
    protected static String EXPORT_AS_IMAGE_MENU_ACTION = "EXPORT_AS_IMAGE";
    protected static String SETTINGS_MENU_ACTION = "PROGRAM_SETTINGS";
    
    /**
     * This method creates and initializes the menu listener for
     * the program menu bar.
     */
    private void createMenuListener() {
    	if ( core == null ) return;
    	
    	/**
    	 * ActionListener for all button events within the dialog.
    	 */
    	menuListener = new ActionListener() {
        	
        	/**
        	 * Action performed: Open file is chosen from the File menu.
        	 */
        	private void menuOpenFile() {
        		System.out.println(LOG_PREFIX + LOG_FILE_LOAD_TRIGGERED);
        		URL file = chooseFile();
        		core.loadUseCases(file);
                dotPanel.updateModel(core.getAnalyzeModel());
                // tree.updateAnalyszeModel(core.getAnalyzeModel());
        	}
        	
        	
         	/**
        	 * Action performed: Export is chosen from the File menu.
        	 */
        	private void menuExport() {
        		System.out.println(LOG_PREFIX + LOG_FILE_LOAD_TRIGGERED);
        		URL file = chooseFile();
        		OutputInterface oi = core.getOutputs().get(0);
        		try{
        			core.output(file,oi,core.getAnalyzeModel());
        		} catch(Exception e){
        			e.printStackTrace();
        		}
        	}
        	
        	/**
        	 * Action performed: Quit is chosen from the File menu.
        	 */
        	private void menuQuit() {
        		if ( !questionDialog(QUIT_QUESTION_HEADER,
        				QUIT_QUESTION).equals(JOptionPane.YES_OPTION) ) return;
        		System.out.println(LOG_PREFIX + LOG_QUIT_PROGRAM_TRIGGERED);
        		System.out.println(LOG_PREFIX + LOG_QUIT_PROGRAM);
        		System.exit(0);
        		// TODO Change to match real shutdown routine.
        	}
        	
        	/**
        	 * Action performed: Clear model action is chosen from the File menu.
        	 */
        	private void menuClearModel() {
            	System.out.println(LOG_PREFIX + LOG_CLEAR_MODEL_TRIGGERED);
            	if ( CLEAR_MODEL_QUESTION_HEADER.length() <= 0 ) {
            		CLEAR_MODEL_QUESTION_HEADER = DEFAULT_QUESTION_HEADER;
            	}
            	if ( questionDialog(CLEAR_MODEL_QUESTION_HEADER,
            			CLEAR_MODEL_QUESTION).equals(JOptionPane.YES_OPTION) ) {
            		core.clearAnalyzeModel();
            	}
        	}
        	
        	/**
        	 * Action performed: Use wants to change program settings.
        	 */
        	private void menuSettings() {
        		System.out.println(LOG_PREFIX + LOG_SETTINGS_TRIGGERED);
        		
                // Launch settingsDialog
                SettingsDialog.showDialog(core);
                
            }
        	
        	private void menuExportAsImage() {
                // FIXME remove comments from method..

/*
                JFileChooser f = new JFileChooser(".");
                f.addChoosableFileFilter( new CustomFileFilter("PNG files", new String[] {"png"}) );
                int returnValue = f.showSaveDialog(GraphicalUI.this);                
                if (returnValue == JFileChooser.APPROVE_OPTION) {
                    File exportFile = f.getSelectedFile();
                    dotPanel.exportImage(exportFile, ExportImageType.PNG);
                }
*/
                dotPanel.exportImage(new File("exportTest.png"), ExportImageType.PNG);
                System.err.println(LOG_PREFIX + LOG_EXPORT_AS_IMAGE_TRIGGERED);
        	}

    		/**
    		 * This method is called when an action is performed and it
    		 * calls the corresponding methods to handle the action. 
    		 */
            public void actionPerformed(ActionEvent e) {
                System.out.println("Action: " + e.getActionCommand());

                if ( OPEN_FILE_MENU_ACTION.equals(e.getActionCommand()) ) {
                	menuOpenFile(); return;
                }
                if ( EXPORT_MENU_ACTION.equals(e.getActionCommand()) ) {
                    menuExport(); return;
                }
                if ( QUIT_MENU_ACTION.equals(e.getActionCommand()) ) {
                    menuQuit(); return;
                }
                if ( CLEAR_MODEL_MENU_ACTION.equals(e.getActionCommand()) ) {
                	menuClearModel(); return;
                }
                if ( SETTINGS_MENU_ACTION.equals(e.getActionCommand()) ) {
                    menuSettings(); return;
                }
                if ( EXPORT_AS_IMAGE_MENU_ACTION.equals(e.getActionCommand()) ) {
                	menuExportAsImage(); return;
                }
            }
        };
    }
    
    /**
     * Constuctor for Graphical UI. This constructor sets up
     * the whole user interface and spawns it to the screen.
     */
    public GraphicalUI(ControlInterface controlInterface) {
    	// Initialize control interface.
        this.core = controlInterface;
        
        // Initialize all panels.
        dotPanel = new DotPanel(core.getAnalyzeModel());
        dotScrollPane = new JScrollPane(dotPanel);
        dotScrollPane.setMinimumSize(new Dimension(250, 400));
        this.setLayout(new BorderLayout());

        // create the tree form AnalyzeModel
        analyzeModelTree = new AnalyzeModelTree(core.getAnalyzeModel());
        modelTreeScrollPane = new JScrollPane(analyzeModelTree);
        modelTreeScrollPane.setMinimumSize(new Dimension(250, 400));
        
        createMenuListener();
        
        // Finalize analyze model tree initialization.
        analyzeModelTree.createMouseListener(core, entityPropertiesDialog);
        analyzeModelTree.setStructurePanel(dotPanel);

        // Create new use case tree.
        this.useCaseTree = new UseCaseTree(core.getUseCaseCollection(), this.core);
        this.useCaseTree.setUseCasePanel(this.useCasePanel);
        this.useCaseTree.setDotPanel(this.dotPanel);
        
        // Put UseCaseTree into a ScrollPane.
        useCaseTreeScrollPane = new JScrollPane(this.useCaseTree);
        
        // Set default exit policy.
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        // Divide screen into four section with SplitPanes.
        JSplitPane treePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, modelTreeScrollPane,
                useCaseTreeScrollPane);
        JSplitPane viewPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, dotScrollPane,
        		(JComponent) useCasePanelScrollPane);
        JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, treePane,
                viewPane);
        
        // Finalize initialization and make window visible.
        this.getContentPane().add(sp, BorderLayout.CENTER);
        this.setJMenuBar(createMenu());
        this.setTitle(PROGRAM_TITLE);
        this.getContentPane().setPreferredSize(new Dimension(1000, 700));
        this.pack();
        this.setVisible(true);
    }
    
    /**
     * This method centers a given dialog to the center of the screen.
     * @param dialog Dialog to be centered.
     */
    public void centerDialog(JDialog dialog) {
    	Dimension screenSize = getToolkit().getScreenSize();
    	Dimension dialogSize = dialog.getSize();
    	Point location = new Point();
    	location.setLocation(
    			Math.max(Math.floor((screenSize.getWidth() - dialogSize.getWidth()) / 2), 0),
    			Math.max(Math.floor((screenSize.getHeight() - dialogSize.getHeight()) / 2), 0));
    	dialog.setLocation(location);
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#printError(java.lang.String, java.lang.String)
     */
    public void printError(String errorMessage, String errorTitle) {
    	final JOptionPane errorPane = new JOptionPane(
    		    errorMessage, JOptionPane.ERROR_MESSAGE);
    	final JDialog errorDialog = new JDialog(new JFrame(), errorTitle, true);
    	
    	// Initialize error message.
    	errorDialog.setContentPane(errorPane);
    	errorDialog.pack();
    	errorDialog.setResizable(false);
    	errorPane.addPropertyChangeListener(
    		new PropertyChangeListener() {
    		    public void propertyChange(PropertyChangeEvent e) {
    		        String property = e.getPropertyName();
    		        
		            if (errorDialog.isVisible() && (e.getSource() == errorPane)
		            		&& (property.equals(JOptionPane.VALUE_PROPERTY))) {
		                errorDialog.setVisible(false);
		            }
		        }
		    }
    	);
    	// Spawn error message.
    	centerDialog(errorDialog);
    	errorDialog.setVisible(true);
    }
    
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#printError(java.lang.String)
     */
    public void printError(String errorMessage) {
    	printError(errorMessage, DEFAULT_ERROR_HEADER);
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#questionDialog(java.lang.String, java.lang.String, int)
     */
    public Object questionDialog(String header, String question, int options) {
    	final JOptionPane questionPane = new JOptionPane(
    		    question, JOptionPane.QUESTION_MESSAGE, options);
    	final JDialog dialog = new JDialog(new JFrame(), header, true);
    	
    	// Initialize question dialog.
    	dialog.setContentPane(questionPane);
    	dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
    	dialog.pack();
    	dialog.setResizable(false);
    	questionPane.addPropertyChangeListener(
    		    new PropertyChangeListener() {
    		        public void propertyChange(PropertyChangeEvent e) {
    		            String prop = e.getPropertyName();

    		            if ( dialog.isVisible() && (e.getSource() == questionPane)
    		            		&& (prop.equals(JOptionPane.VALUE_PROPERTY)) ) {
    		                dialog.setVisible(false);
    		            }
    		        }
    		    }
    	);
    	
    	// Spawn question dialog and return user's choice.
    	centerDialog(dialog);
    	dialog.setVisible(true);
    	return questionPane.getValue();
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#questionDialog(java.lang.String)
     */
    public Object questionDialog(String question) {
    	return questionDialog(DEFAULT_QUESTION_HEADER, question,
    			JOptionPane.YES_NO_OPTION);
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#questionDialog(java.lang.String, java.lang.String)
     */
    public Object questionDialog(String header, String question) {
    	return questionDialog(DEFAULT_QUESTION_HEADER, question,
    			JOptionPane.YES_NO_OPTION);
    }

    /**
     * Method for creating a menu bar to the GUI.
     * @return Program menu bar as a JMenuBar object. 
     */
    private JMenuBar createMenu() {

        /* Menu will look something like this: 
         * +------+----------+-------- 
         * | File | Program  |
         * =========================== 
         * | Open | Settings | 
         * +------+----------+ 
         * | Exit | 
         * +------+
         */

        JMenuBar result = new JMenuBar();
        
        // FILE menu.        
        JMenu menu = new JMenu(FILE_MENU_CAPTION);

        JMenuItem menuItem = new JMenuItem(OPEN_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(OPEN_FILE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(EXPORT_AS_IMAGE_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(EXPORT_AS_IMAGE_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(EXPORT_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(EXPORT_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(CLEAR_MODEL_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(CLEAR_MODEL_MENU_ACTION);
        menu.add(menuItem);
        menuItem = new JMenuItem(QUIT_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(QUIT_MENU_ACTION);
        menu.add(menuItem);
        result.add(menu);
        
        // PROGRAM menu.
        menu = new JMenu(PROGRAM_MENU_CAPTION);

        menuItem = new JMenuItem(SETTINGS_MENU_CAPTION);
        menuItem.addActionListener(menuListener);
        menuItem.setActionCommand(SETTINGS_MENU_ACTION);
        menu.add(menuItem);
        result.add(menu);
        
        // Menu bar successfully created!
        return result;
    }

    /**
     * Method for spawning a file chooser dialog.
     * @return The selected file as a string or null if no
     *   file was selected.
     */
    private URL chooseFile() {
    	JFileChooser fileChooser = new JFileChooser(".");
        int choice = fileChooser.showOpenDialog(this);
        
        URL result = null;
        
        if ( choice == JFileChooser.APPROVE_OPTION ) {
            try {
                result = fileChooser.getSelectedFile().toURL();
            } catch (Exception e) {
                System.err.println(e.getMessage());
                System.err.println(e.getStackTrace().toString());
            }
        }
    	return result;
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#exportDone()
     */
    public void exportDone() {
        System.out.println(LOG_PREFIX + LOG_EXPORT_DONE_MESSAGE);
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#analyzeModelLoaded()
     */
    public void analyzeModelLoaded() {
        System.out.println(LOG_PREFIX + LOG_ANALYZE_MODEL_LOADED);
    }
    
    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#useCasesLoaded()
     */
    public void useCasesLoaded() {
        System.out.println(LOG_PREFIX + LOG_USE_CASES_LOADED);
        UseCaseCollection collection = core.getUseCaseCollection();
        try {
            if (collection.getUseCaseCount() > 0) {
                // Ask user which use case he or she desires to load.
                //			UseCase uc = useCaseSelector.selectUseCase(collection);
                //			core.addToAnalyzeModel(uc);

            } else {
                throw new
                  Exception(LOG_USE_CASE_LOADING_EXCEPTION);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#useCaseAdded()
     */
    public void useCaseAdded() {
        System.out.println(LOG_PREFIX + LOG_USE_CASES_ADDED);
    }

    /* (non-Javadoc)
     * @see ucot.ui.UIInterface#setControlInterface(ucot.core.ControlInterface)
     */
    public void setControlInterface(ControlInterface a) {
        core = a;
    }
}
