package ucot.ui.gui;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.List;
import java.util.Properties;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;

import ucot.core.ControlInterface;
import ucot.heuristic.HeuristicInterface;
import ucot.input.InputCollection;
import ucot.input.InputInterface;
import ucot.parser.ParserInterface;

public class SettingsDialog extends JDialog {

    private ControlInterface core;
    
    private ActionListener buttonListener;
    
    //
    // Dialog Components
    private JTabbedPane tabbedPane;
    private JPanel      generalPanel;
    private JPanel      inputPanel;
    private JPanel      parserPanel;
    private JPanel      heuristicPanel;
    private JPanel      graphPanel;
    private JPanel      outputPanel;
    
    private JButton     okButton; 
    private JButton     cancelButton; 
    private JButton     applyButton; 
    
    private JComboBox   parserComboBox;
    private JComboBox   heuristicComboBox;
    
    private JTextField  dotPathTextField;
    // Dialog Components
    //
    
    // ActionCommands for buttons
    private final String BUTTON_OK_CLICKED = "BUTTON_OK_CLICKED";
    private final String BUTTON_CANCEL_CLICKED = "BUTTON_CANCEL_CLICKED";
    private final String BUTTON_APPLY_CLICKED = "BUTTON_APPLY_CLICKED";
    
    
    public SettingsDialog(Frame owner, ControlInterface core) throws HeadlessException {
        super(owner, "Settings", true);
        
        this.core = core;

        // Create panel where tabbedPane is located..
        JPanel topPanel = new JPanel();
        this.setLayout( new BorderLayout() );
        this.getContentPane().add(topPanel, BorderLayout.CENTER);
        
        
        // Create tabbedPane
        this.tabbedPane = new JTabbedPane();
        
        // set up panels for tabs
        createGeneralTab();
        createInputTab();
        createParserTab();
        createHeuristicTab();
        createGraphTab();
        createOutputTab();
        // set panels into tabbedPane
        
        this.tabbedPane.addTab("General", this.generalPanel);
        this.tabbedPane.addTab("Input", this.inputPanel);
        this.tabbedPane.addTab("Parser", this.parserPanel);
        this.tabbedPane.addTab("Heuristic", this.heuristicPanel);
        this.tabbedPane.addTab("Graph", this.graphPanel);
        this.tabbedPane.addTab("Output", this.outputPanel);
        
        
        // add tabbedPane into topPanel
        topPanel.setLayout(new BorderLayout());
        topPanel.add(this.tabbedPane, BorderLayout.CENTER);
        
        
        // add buttonPanel and buttons
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.TRAILING));
        
        // initialize actionlistener for buttons
        this.createButtonListener();
        
        this.okButton = new JButton("OK");
        this.okButton.setActionCommand(BUTTON_OK_CLICKED);
        this.okButton.addActionListener(this.buttonListener);

        this.cancelButton = new JButton("Cancel");
        this.cancelButton.setActionCommand(BUTTON_CANCEL_CLICKED);
        this.cancelButton.addActionListener(this.buttonListener);
        
        this.applyButton = new JButton("Apply");
        this.applyButton.setActionCommand(BUTTON_APPLY_CLICKED);
        this.applyButton.addActionListener(this.buttonListener);
        
        buttonPanel.add(okButton);
        buttonPanel.add(cancelButton);
        buttonPanel.add(applyButton);
        
        topPanel.add(buttonPanel, BorderLayout.SOUTH);
        
        
        this.pack();
    }
    
    /**
     * Initialize panel for general settings, like 
     * default parser and heuristics etc.
     *
     */
    private void createGeneralTab() {
        
        // create panel and add layout for it
        this.generalPanel = new JPanel();     
        this.generalPanel.setLayout(new BoxLayout(generalPanel, BoxLayout.Y_AXIS));
        
        //
        // Default parser
        JPanel defaultParserPanel = new JPanel();
        defaultParserPanel.setLayout( getCustomFlowLayout() );
        defaultParserPanel.add( new JLabel("Default parser") );
        
        // create ComboBox for selecting default parser
        this.parserComboBox = new JComboBox();
        List<ParserInterface> parsers = this.core.getParsers();
        for (ParserInterface parser : parsers) {
            this.parserComboBox.addItem(parser);
        }
        this.parserComboBox.setSelectedItem( this.core.getCurrentParser() );
        defaultParserPanel.add( this.parserComboBox );        
        // Default parser
        //
        
        
        //
        // Default heuristic
        JPanel defaultHeuristicPanel = new JPanel();
        defaultHeuristicPanel.setLayout( getCustomFlowLayout() );
        defaultHeuristicPanel.add( new JLabel("Default heuristic") );
        
        // create ComboBox for selecting default heuristic
        this.heuristicComboBox = new JComboBox();
        List<HeuristicInterface> heuristics = this.core.getHeuristics();
        for (HeuristicInterface heuristic : heuristics) {
            this.heuristicComboBox.addItem(heuristic);
        }
        this.heuristicComboBox.setSelectedItem(this.core.getCurrentHeuristic());
        defaultHeuristicPanel.add( this.heuristicComboBox );
        // Default heuristic
        //
        
        
        this.generalPanel.add(defaultParserPanel);
        this.generalPanel.add(defaultHeuristicPanel);
    }
    
    /**
     * Initialize panel for input related settings,
     * like settings for each adapter etc.
     *
     */
    private void createInputTab() {
        this.inputPanel = new JPanel();
        this.inputPanel.setLayout(new BorderLayout());
        
        //
        // Panel for combobox of input adapter 
        // and settings table
        JPanel inputAdapterPanel = new JPanel();
        inputAdapterPanel.setLayout(new BorderLayout());
        inputAdapterPanel.setBorder(BorderFactory.createTitledBorder("InputAdapter settings"));
        
        // create ComboBox fot selecting input to edit
        JPanel comboBoxPanel = new JPanel();
        comboBoxPanel.setLayout(new FlowLayout());
        JComboBox inputSelectComboBox = new JComboBox();
        InputCollection inputs = this.core.getInputs();
        for (int i = 0; i < inputs.getInputCount(); i++) {
            inputSelectComboBox.addItem(inputs.getInput(i));
        }
        comboBoxPanel.add(inputSelectComboBox);
        inputAdapterPanel.add(comboBoxPanel, BorderLayout.NORTH);
        final JPanel tablePanel = new JPanel();
        tablePanel.setLayout(new BorderLayout());
        tablePanel.add(getTableOfProperties(null), BorderLayout.CENTER);
        inputAdapterPanel.add(tablePanel, BorderLayout.CENTER);
        inputSelectComboBox.addActionListener( new ActionListener() {
        
            public void actionPerformed(ActionEvent e) {
                JComboBox c = (JComboBox) e.getSource();
                InputInterface input = (InputInterface) c.getSelectedItem();
                
                tablePanel.removeAll();
                
                tablePanel.add(getTableOfProperties(null), BorderLayout.CENTER);
                validate();
            }
        
        } );
        // Panel
        //
        
        
        //
        // Panel for textfield of characters to 
        // discard in input.
        JPanel discardCharsPanel = new JPanel();
        discardCharsPanel.setLayout(new FlowLayout());
        discardCharsPanel.setBorder(BorderFactory.createTitledBorder("Discard characters in input"));
        JTextField discardedCharsTextField = new JTextField(17);
        discardCharsPanel.add(discardedCharsTextField);
        // Panel
        //
        
        
        this.inputPanel.add(inputAdapterPanel, BorderLayout.CENTER);
        this.inputPanel.add(discardCharsPanel, BorderLayout.SOUTH);
    }
    
    /**
     * Initialize panel for parser related settings
     *
     */
    private void createParserTab() {
        this.parserPanel = new JPanel();
        this.parserPanel.setLayout(new BorderLayout());

        //
        // Panel for combobox of input adapter 
        // and settings table
        JPanel inputAdapterPanel = new JPanel();
        inputAdapterPanel.setLayout(new BorderLayout());
        inputAdapterPanel.setBorder(BorderFactory.createTitledBorder("Parser settings"));
        
        // create ComboBox fot selecting input to edit
        JPanel comboBoxPanel = new JPanel();
        comboBoxPanel.setLayout(new FlowLayout());
        JComboBox inputSelectComboBox = new JComboBox();
        List<ParserInterface> parsers = this.core.getParsers();
        for (ParserInterface parser : parsers) {
            inputSelectComboBox.addItem(parser);
        }
        comboBoxPanel.add(inputSelectComboBox);
        inputAdapterPanel.add(comboBoxPanel, BorderLayout.NORTH);
        final JPanel tablePanel = new JPanel();
        tablePanel.setLayout(new BorderLayout());
        tablePanel.add(getTableOfProperties(null), BorderLayout.CENTER);
        inputAdapterPanel.add(tablePanel, BorderLayout.CENTER);
        inputSelectComboBox.addActionListener( new ActionListener() {
        
            public void actionPerformed(ActionEvent e) {
                JComboBox c = (JComboBox) e.getSource();
                
                tablePanel.removeAll();
                
                tablePanel.add(getTableOfProperties(null), BorderLayout.CENTER);
                validate();
            }
        
        } );
        // Panel
        //
        
        this.parserPanel.add(inputAdapterPanel);
    }
    
    /**
     * Initialize panel for heuristic related settings
     *
     */
    private void createHeuristicTab() {
        this.heuristicPanel = new JPanel();
        this.heuristicPanel.setLayout(new BorderLayout());

        //
        // Panel for combobox of input adapter 
        // and settings table
        JPanel inputAdapterPanel = new JPanel();
        inputAdapterPanel.setLayout(new BorderLayout());
        inputAdapterPanel.setBorder(BorderFactory.createTitledBorder("Heuristics settings"));
        
        // create ComboBox fot selecting input to edit
        JPanel comboBoxPanel = new JPanel();
        comboBoxPanel.setLayout(new FlowLayout());
        JComboBox inputSelectComboBox = new JComboBox();
        List<HeuristicInterface> heuristics = this.core.getHeuristics();
        for (HeuristicInterface heuristic : heuristics) {
            inputSelectComboBox.addItem(heuristic);
        }
        comboBoxPanel.add(inputSelectComboBox);
        inputAdapterPanel.add(comboBoxPanel, BorderLayout.NORTH);
        final JPanel tablePanel = new JPanel();
        tablePanel.setLayout(new BorderLayout());
        tablePanel.add(getTableOfProperties(null), BorderLayout.CENTER);
        inputAdapterPanel.add(tablePanel, BorderLayout.CENTER);
        inputSelectComboBox.addActionListener( new ActionListener() {
        
            public void actionPerformed(ActionEvent e) {
                JComboBox c = (JComboBox) e.getSource();
                
                tablePanel.removeAll();
                
                tablePanel.add(getTableOfProperties(null), BorderLayout.CENTER);
                validate();
            }
        
        } );
        // Panel
        //
        
        this.heuristicPanel.add(inputAdapterPanel);
    }

    /**
     * Initialize panel for settings related
     * drawing graphs and DotPanel
     *
     */
    private void createGraphTab() {
        this.graphPanel = new JPanel();
        this.graphPanel.setLayout(new BoxLayout(graphPanel, BoxLayout.Y_AXIS));
    
        //
        // path to dot.exe
        JPanel dotPathPanel = new JPanel();
        dotPathPanel.setLayout( getCustomFlowLayout() );
        dotPathPanel.add( new JLabel("Dot location") );
        
        this.dotPathTextField = new JTextField(17);
        dotPathPanel.add( this.dotPathTextField );
        
        JButton searchDotButton = new JButton("Browse");
        // add actionListener for searhDotButton
        searchDotButton.addActionListener( new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFileChooser fc = new JFileChooser(".");
                int returnValue = fc.showOpenDialog(null);
                
                // if user pressed OK, set choosen file as dot.exe location
                if (returnValue == JFileChooser.APPROVE_OPTION) {
                    File result = fc.getSelectedFile();
                    dotPathTextField.setText(result.getAbsolutePath());
                }
            }
        } );
        dotPathPanel.add(searchDotButton);
        // path to dot.exe
        //

        
        this.graphPanel.add(dotPathPanel);
    }
    
    /**
     * Initialize panel for ouput related
     * settings
     *
     */
    private void createOutputTab() {
        this.outputPanel = new JPanel();
    }
    
    /**
     * Method initializes ActionListener for buttons
     *
     */
    private void createButtonListener() {
        
        // create new actionlistener for buttons
        this.buttonListener = new ActionListener() {
        
            /**
             * User just clicked OK button 
             * now we should save settings and
             * close this dialog.
             *
             */
            private void buttonOKClicked() {
                
                // cause we just save settings in
                // buttonApplyclicked method, we 
                // reuse it here.
                buttonApplyClicked();
                
                // close the dialog
                setVisible(false);
            }
            
            /**
             * User just clicked Cancel
             * button and we just close 
             * the dialog without saving
             * anything.
             *
             */
            private void buttonCancelClicked() {
                // close the dialog
                setVisible(false);
            }
            
            /**
             * User clicked Apply and we 
             * just save the settings.
             *
             */
            private void buttonApplyClicked() {
                
                //
                // Save settings here....
                //
                
            }
            
            public void actionPerformed(ActionEvent e) {
                
                // Check what just happened and act based on it..
                if ( BUTTON_OK_CLICKED.equals(e.getActionCommand()) ) {
                    buttonOKClicked();
                    return;
                }
                if ( BUTTON_CANCEL_CLICKED.equals(e.getActionCommand()) ) {
                    buttonCancelClicked();
                    return;                    
                }
                if ( BUTTON_APPLY_CLICKED.equals(e.getActionCommand()) ) {
                    buttonApplyClicked();
                    return;
                }
            }
        
        };
        
    }

    
    /**
     * Static method to create and show SettingsDialog.
     *
     */
    public static void showDialog(ControlInterface core) {
       
        // create new SettingsDialog
        SettingsDialog dialog = new SettingsDialog(null, core);
        
        // show created dialog
        dialog.setVisible(true);
        
        // erease pointer to dialog
        dialog = null;
    }
    
    /**
     * Returns custom FlowLayout. Horizontal and vertical gaps are
     * bit wider than in normal FlowLayout to make layout look better.
     * @return FlowLayout
     */
    private FlowLayout getCustomFlowLayout() {
        return new FlowLayout(FlowLayout.CENTER, 10, 15);
    }
    
    private JTable getTableOfProperties(Properties properties) {
        
        JTable result = new JTable(new String[][]{{"property1", "value1"}, {"property2", "value2"}, {"property3", "value3"}, {"property4", "value4"}}, new String[]{"Key", "Value"});
        
        
        return result;
    }
    
}
